/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.View = Class.$define("APT.View");

APT.ActionSheet = Class.$define("APT.ActionSheet", APT.View, {
    __init__: function() {
        this._delegate = null;
        this._title = "";
        this._cancelButtonIndex = -1;
        this._destructiveButtonIndex = -1;
        this._firstOtherButtonIndex = -1;
        this._buttons = new Array();
        this.$super();
    },

    $copy: function() {
        var copyObject = new APT.ActionSheet();
        copyObject._delegate = this._delegate;
        copyObject._title = this._title;
        copyObject._cancelButtonIndex = this._cancelButtonIndex;
        copyObject._destructiveButtonIndex = this._destructiveButtonIndex;
        copyObject._firstOtherButtonIndex = this._firstOtherButtonIndex;
        copyObject._buttons = this._buttons;
        return copyObject;
    },

    getRenderMarkup: function() {
        return $(APT.Global.toStaticHTML("<div><div data-role='dialog'></div></div>"));
    },

    setJQElement: function(jqelement) {
        var screenObject = APT.Screen.CreateMainScreen();
        var view = new APT.View().initWithFrame(screenObject.applicationFrame());
        view.setBackgroundColor(APT.Color.New([1 / 2, 1 / 2, 1 / 2, 0.8]));
        var contentDiv = jqelement.trigger("create").children(1);
        contentDiv.append(this.getObjTitle(""));
        contentDiv.css("display", "inline-block");
        contentDiv.css("background", "black");
        contentDiv.css("min-height", "150px");
        contentDiv.css("width", "240px");
        contentDiv.css("text-align", APT.TextAlignment.toCSSTextAlign(APT.TextAlignment.Center));
        contentDiv.addClass("ui-dialog-contain ui-corner-all ui-overlay-shadow");
        contentDiv.addClass("ui-content-dialog");
        contentDiv.css("position", "fixed");
        contentDiv.css("top", "55%");
        contentDiv.css("left", "50%");
        contentDiv.css("margin-left", "-120px");
        contentDiv.css("margin-top", "-10px");
        view.jQElement().append(contentDiv);
        this._jqElement = view.jQElement();
        this.hide(null);
    },

    getObjTitle: function(aTitle) {
        var jqo;
        jqo = $("<div><h1 class='ui-actionSheet-title'>" + aTitle + "</h1></div>").trigger("create").children(1);
        return jqo;
    },

    getObjBtnCancel: function(aCancelTitle) {
        var btnCancel = APT.Button.buttonWithType(APT.ButtonType.RoundedRect);
        btnCancel.setTitleColor(APT.Color.White);
        btnCancel.setTitle(aCancelTitle);
        btnCancel.jQElement().attr("id", aCancelTitle);
        btnCancel.setBackgroundColor(APT.Color.DarkGray);
        btnCancel.addTarget_action_forControlEvents(this, "hide", APT.ControlEvent.vmousedown);
        this._buttons.push(btnCancel.jQElement());
        return btnCancel.jQElement();
    },

    getObjBtnDestructive: function(aDestructiveTitle) {
        var btnDestructive = APT.Button.buttonWithType(APT.ButtonType.RoundedRect);
        btnDestructive.setTitleColor(APT.Color.White);
        btnDestructive.setTitle(aDestructiveTitle);
        btnDestructive.jQElement().attr("id", aDestructiveTitle);
        btnDestructive.setBackgroundColor(APT.Color.Red);
        btnDestructive.addTarget_action_forControlEvents(this, "hide", APT.ControlEvent.vmousedown);
        this._buttons.push(btnDestructive.jQElement());
        return btnDestructive.jQElement();
    },

    getObjBtnOthers: function(aOtherTitle) {
        var btnOther = APT.Button.buttonWithType(APT.ButtonType.RoundedRect);
        btnOther.setTitle(aOtherTitle);
        btnOther.jQElement().attr("id", aOtherTitle);
        btnOther.addTarget_action_forControlEvents(this, "hide", APT.ControlEvent.vmousedown);
        this._buttons.push(btnOther.jQElement());
        return btnOther.jQElement();
    },

    initWithTitle_delegate_cancelButtonTitle_destructiveButtonTitle_otherButtonTitles: function(aTitle, aDelegate, aCancelButtonTitle, aDestructiveButtonTitle, aOtherButton) {
        this._delegate = aDelegate;

        if (aTitle !== null)
        {
            this.setTitle(aTitle);
        }

        var index = 0;
        if (aDestructiveButtonTitle !== null)
        {
            this._destructiveButtonIndex = index++;
            var contentChild = this.jQElement().find(".ui-content-dialog");
            contentChild.append(this.getObjBtnDestructive(aDestructiveButtonTitle));
        }

        if (arguments.length > 4)
        {
            for (var n = 4; n < arguments.length; n++)
            {
                var otherBtnTitle = arguments[n];
                if (otherBtnTitle !== null)
                {
                    if (n === 4)
                    {
                        this._firstOtherButtonIndex = index;
                    }

                    index++;
                    var contentChild = this.jQElement().find(".ui-content-dialog");
                    contentChild.append(this.getObjBtnOthers(otherBtnTitle));
                }
            }
        }

        if (aCancelButtonTitle !== null)
        {
            this._cancelButtonIndex = index;
            var contentChild = this.jQElement().find(".ui-content-dialog");
            contentChild.append(this.getObjBtnCancel(aCancelButtonTitle));
        }

        return this;
    },

    hide: function(ev) {
        var index = -1;

        if (ev !== null)
        {
            index = this.getIndexForId((ev).jQElement().attr("id"));
        }

        if (this._delegate !== null && index !== -1)
        {
            if (this._delegate["actionSheet_clickedButtonAtIndex"] instanceof Function)
            {
                this._delegate.actionSheet_clickedButtonAtIndex(this, index);
            }

            if (this._delegate["actionSheet_willDismissWithButtonIndex"] instanceof Function)
            {
                this._delegate.actionSheet_willDismissWithButtonIndex(this, index);
            }
        }

        this.jQElement().hide();
        if (this._delegate !== null && index !== -1 && this._delegate["actionSheet_didDismissWithButtonIndex"] instanceof Function)
        {
            this._delegate.actionSheet_didDismissWithButtonIndex(this, index);
        }
    },

    getIndexForId: function(btnId) {
        var result = -1;
        for (var i = 0; i < this._buttons.length; i++)
        {
            var tempObj = this._buttons[i];
            var tempStr = tempObj.attr("id");
            if (tempStr.apt_isEqual(btnId))
            {
                result = i;
                break;
            }
        }

        return result;
    },

    setTitle: function(aTitle) {
        this._title = aTitle;
        var title = this.jQElement().find(".ui-actionSheet-title");
        title.html(aTitle.valueOf());
    },

    title: function() {
        return this._title;
    },

    showInView: function(view) {
        view.addSubview(this);
        this.jQElement().show();
    },

    viewWillAppear: function(animated) {
        if (this._delegate !== null && this._delegate["willPresentActionSheet"] instanceof Function)
        {
            this._delegate.willPresentActionSheet(this);
        }

        this.$super(animated);
    },

    viewDidAppear: function(animated) {
        if (this._delegate !== null && this._delegate["didPresentActionSheet"] instanceof Function)
        {
            this._delegate.didPresentActionSheet(this);
        }

        this.$super(animated);
    },

    numberOfButtons: function() {
        return this._buttons.length;
    },

    cancelButtonIndex: function() {
        return this._cancelButtonIndex;
    },

    setCancelButtonIndex: function(aIndex) {
        this._cancelButtonIndex = aIndex;
    },

    destructiveButtonIndex: function() {
        return this._destructiveButtonIndex;
    },

    setDestructiveButtonIndex: function(aIndex) {
        this._destructiveButtonIndex = aIndex;
    },

    firstOtherButtonIndex: function() {
        return this._firstOtherButtonIndex;
    },

    setFirstOtherButtonIndex: function(aIndex) {
        this._firstOtherButtonIndex = aIndex;
    },

    setDelegate: function(aDelegate) {
        this._delegate = aDelegate;
    },

    delegate: function() {
        return this._delegate;
    },

    buttonTitleAtIndex: function(index) {
        if (index >= 0 && index <= this._buttons.length)
        {
            (this._buttons[index]).title();
        }

        return null;
    },

});


