/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.View = Class.$define("APT.View");

APT.AlertView = Class.$define("APT.AlertView", APT.View, {
    __init__: function() {
        this._delegate = null;
        this._arrayButtons = new Array();
        this.$super();
    },

    $copy: function() {
        var copyObject = new APT.AlertView();
        copyObject._delegate = this._delegate;
        copyObject._arrayButtons = this._arrayButtons;
        return copyObject;
    },

    getRenderMarkup: function() {
        return $(APT.Global.toStaticHTML("<div><div data-role='dialog'></div></div>"));
    },

    setJQElement: function(jqelement) {
        var screenObject = APT.Screen.CreateMainScreen();
        var view = new APT.View().initWithFrame(screenObject.applicationFrame());
        view.setBackgroundColor(APT.Color.New([1 / 2, 1 / 2, 1 / 2, 0.8]));
        view.jQElement().css("z-index", 123456789);
        var contentDiv = jqelement.trigger("create").children(1);
        contentDiv.append(this.getObjTitle(""));
        contentDiv.append(this.getObjMessage(""));
        contentDiv.css("display", "inline-block");
        contentDiv.css("background-color", "white");
        contentDiv.css("min-height", "150px");
        contentDiv.css("width", "240px");
        contentDiv.css("text-align", APT.TextAlignment.toCSSTextAlign(APT.TextAlignment.Center));
        contentDiv.addClass("ui-dialog-contain ui-corner-all ui-overlay-shadow");
        contentDiv.addClass("ui-content-dialog");
        contentDiv.css("position", "fixed");
        contentDiv.css("top", "40%");
        contentDiv.css("left", "50%");
        contentDiv.css("margin-left", "-120px");
        contentDiv.css("margin-top", "-50px");
        view.jQElement().append(contentDiv);
        this._jqElement = view.jQElement();
        this.hide();
    },

    getObjTitle: function(aTitle) {
        var jqo;
        jqo = $("<div><h1 class='ui-alertview-title'>" + aTitle + "</h1></div>").trigger("create").children(1);
        return jqo;
    },

    getObjMessage: function(aMessage) {
        var jqo;
        jqo = $("<div><p class='ui-alertview-message'>" + aMessage + "</p></div>").trigger("create").children(1);
        return jqo;
    },

    getObjBtnCancel: function(aCancelTitle) {
        var btnCancel = APT.Button.buttonWithType(APT.ButtonType.Custom);
        btnCancel.setTitle(aCancelTitle);
        btnCancel.jQElement().attr("id", aCancelTitle);
        btnCancel.addTarget_action_forControlEvents(this, "hideWithEvent", APT.ControlEvent.vmouseup);
        this._arrayButtons.push(btnCancel.jQElement());
        return btnCancel.jQElement();
    },

    getObjBtnOthers: function(aOtherTitle) {
        var btnOther = APT.Button.buttonWithType(APT.ButtonType.Custom);
        btnOther.setTitle(aOtherTitle);
        btnOther.jQElement().attr("id", aOtherTitle);
        btnOther.addTarget_action_forControlEvents(this, "hideWithEvent", APT.ControlEvent.vmouseup);
        this._arrayButtons.push(btnOther.jQElement());
        return btnOther.jQElement();
    },

    getIndexForId: function(btnId) {
        var result = -1;
        for (var i = 0; i < this._arrayButtons.length; i++)
        {
            var tempObj = this._arrayButtons[i];
            var tempStr = tempObj.attr("id");
            if (tempStr.apt_isEqual(btnId))
            {
                result = i;
                break;
            }
        }

        return result;
    },

    initWithTitle_message_delegate_cancelButtonTitle_otherButtonTitles: function(aTitle, aMessage, aDelegate, cancelBtnTitle, otherBtnTitles) {
        this.setDelegate(aDelegate);
        if (aTitle !== null)
        {
            this.setTitle(aTitle);
        }

        if (aMessage !== null)
        {
            this.setMessage(aMessage);
        }

        if (arguments.length > 4)
        {
            for (var n = 4; n < arguments.length; n++)
            {
                var otherBtnTitle = arguments[n];
                if (otherBtnTitle !== null)
                {
                    var contentChild = this.jQElement().find(".ui-content-dialog");
                    contentChild.append(this.getObjBtnOthers(otherBtnTitle));
                }
            }
        }

        if (cancelBtnTitle !== null)
        {
            var contentChild = this.jQElement().find(".ui-content-dialog");
            contentChild.append(this.getObjBtnCancel(cancelBtnTitle));
        }

        return this;
    },

    show: function() {
        if (this._delegate !== null && this._delegate["willPresentAlertView"] instanceof Function)
        {
            this._delegate.willPresentAlertView(this);
        }

        $("body").append(this.jQElement());
        this.jQElement().show();
        if (this._delegate !== null && this._delegate["didPresentAlertView"] instanceof Function)
        {
            this._delegate.didPresentAlertView(this);
        }

        if (APT.AlertView._alertVisible === null)
        {
            APT.AlertView._alertVisible = this;
        }
    },

    hideWithEvent: function(ev) {
        var index = -1;
        if (ev !== null)
        {
            index = this.getIndexForId((ev).jQElement().attr("id"));
        }

        if (this._delegate !== null && index !== -1)
        {
            if (this._delegate["alertView_clickedButtonAtIndex"] instanceof Function)
            {
                this._delegate.alertView_clickedButtonAtIndex(this, index);
            }

            if (this._delegate["alertView_willDismissWithButtonIndex"] instanceof Function)
            {
                this._delegate.alertView_willDismissWithButtonIndex(this, index);
            }
        }

        this.hide();
        if (this._delegate !== null && index !== -1 && this._delegate["alertView_didDismissWithButtonIndex"] instanceof Function)
        {
            this._delegate.alertView_didDismissWithButtonIndex(this, index);
        }
    },

    hide: function() {
        this.jQElement().hide();
        this.jQElement().detach();
    },

    isVisible: function() {
        if (APT.AlertView._alertVisible === this)
        {
            return true;
        }
        else
        {
            return false;
        }
    },

    setTitle: function(aTitle) {
        var titleChild = this.jQElement().find(".ui-alertview-title");
        titleChild.html(aTitle.valueOf());
    },

    title: function() {
        var titleChild = this.jQElement().find(".ui-alertview-title");
        return titleChild.html();
    },

    setMessage: function(message) {
        var msgChild = this.jQElement().find(".ui-alertview-message");
        msgChild.html(message.valueOf());
    },

    message: function() {
        var msgChild = this.jQElement().find(".ui-alertview-message");
        return msgChild.html();
    },

    setDelegate: function(aDelegate) {
        this._delegate = aDelegate;
    },

    delegate: function() {
        return this._delegate;
    },

    numberOfButtons: function() {
        return this._arrayButtons.length;
    },

    viewWillAppear: function(animated) {
        if (this._delegate !== null && this._delegate["willPresentAlertView"] instanceof Function)
        {
            this._delegate.willPresentAlertView(this);
        }

        this.$super(animated);
    },

    viewDidAppear: function(animated) {
        if (this._delegate !== null && this._delegate["didPresentAlertView"] instanceof Function)
        {
            this._delegate.didPresentAlertView(this);
        }

        this.$super(animated);
    },

});

APT.AlertView._alertVisible = null;


