/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.BarItem = Class.$define("APT.BarItem");

APT.BarButtonItem = Class.$define("APT.BarButtonItem", APT.BarItem, {
    __init__: function() {
        this._defaultView = null;
        this._target = null;
        this._action = null;
        this._style = APT.BarButtonItemStyle.Plain;
        this.$super();
        this._defaultView = new APT.BarButtonDefault();
        this._defaultView.addTarget_action_forControlEvents(this, "triggerTargetAction", APT.ControlEvent.vmouseup);
    },

    $copy: function() {
        var copyObject = new APT.BarButtonItem();
        copyObject._defaultView = this._defaultView;
        copyObject._target = this._target;
        copyObject._action = this._action;
        copyObject._style = this._style;
        return copyObject;
    },

    initWithTitle_target_action: function(aTitle, aTarget, anAction, aStyle) {
        this.setTitle(aTitle);
        this.setTarget(aTarget);
        this.setAction(anAction);
        this.setStyle(aStyle);
        return this;
    },

    initWithBarButtonSystemItem_target_action: function(aButtonSystemItem, aTarget, anAction) {
        this.setBarButtonSystemItem(aButtonSystemItem);
        this.setTarget(aTarget);
        this.setAction(anAction);
        return this;
    },

    defaultView: function() {
        return this._defaultView;
    },

    setDefaultView: function(defaultView) {
        this._defaultView = defaultView;
    },

    setTitle: function(aTitle) {
        this._defaultView.setText(aTitle);
    },

    title: function() {
        return this._defaultView.text();
    },

    setTag: function(tag) {
        this._defaultView.setTag(tag);
    },

    tag: function() {
        return this._defaultView.tag();
    },

    setTarget: function(aTarget) {
        this._target = aTarget;
    },

    target: function() {
        return this._target;
    },

    setAction: function(anAction) {
        this._action = anAction;
    },

    action: function() {
        return this._action;
    },

    setEnabled: function(enabled) {
        this._defaultView.setEnabled(enabled);
    },

    isEnabled: function() {
        return this._defaultView.isEnabled();
    },

    setStyle: function(aStyle) {
        this._style = aStyle;
        if (aStyle === APT.BarButtonItemStyle.Plain)
        {
            this.defaultView().setBackgroundColor(APT.Color.Transparent);

            this.defaultView().jQElement().css("border-top-color", "transparent");
            this.defaultView().jQElement().css("border-left-color", "transparent");
            this.defaultView().jQElement().css("border-right-color", "transparent");
            this.defaultView().jQElement().css("border-bottom-color", "transparent");
            this.defaultView().jQElement().css("color", "white");
        }
        else
        {
            if (aStyle === APT.BarButtonItemStyle.Done)
            {
                this.defaultView().jQElement().buttonMarkup({"theme" : "b"});
            }
        }
    },

    style: function() {
        return this._style;
    },

    setBarButtonSystemItem: function(aBarButtonSystemItem) {
        this._style = aBarButtonSystemItem;
        switch(aBarButtonSystemItem)
        {
            case APT.BarButtonSystemItem.Done:
                this.setTitle("Done");
                break;
            case APT.BarButtonSystemItem.Cancel:
                this.setTitle("Cancel");
                break;
            case APT.BarButtonSystemItem.Edit:
                this.setTitle("Edit");
                break;
            case APT.BarButtonSystemItem.Save:
                this.setTitle("Save");
                break;
            case APT.BarButtonSystemItem.Add:
                this.setTitle("Add");
                break;
            case APT.BarButtonSystemItem.FlexibleSpace:
                this.setTitle(new APT.View());
                break;
            case APT.BarButtonSystemItem.FixedSpace:
                this.setTitle(new APT.View());
                break;
            case APT.BarButtonSystemItem.Compose:
                this.setTitle("Compose");
                break;
            case APT.BarButtonSystemItem.Reply:
                this.setTitle("Reply");
                break;
            case APT.BarButtonSystemItem.Action:
                this.setTitle("Action");
                break;
            case APT.BarButtonSystemItem.Organize:
                this.setTitle("Organize");
                break;
            case APT.BarButtonSystemItem.Bookmarks:
                this.setTitle("Bookmarks");
                break;
            case APT.BarButtonSystemItem.Search:
                this.setTitle("Search");
                break;
            case APT.BarButtonSystemItem.Refresh:
                this.setTitle("Refresh");
                break;
            case APT.BarButtonSystemItem.Stop:
                this.setTitle("Stop");
                break;
            case APT.BarButtonSystemItem.Camera:
                this.setTitle("Camera");
                break;
            case APT.BarButtonSystemItem.Trash:
                this.setTitle("Trash");
                break;
            case APT.BarButtonSystemItem.Play:
                this.setTitle("Play");
                break;
            case APT.BarButtonSystemItem.Pause:
                this.setTitle("Pause");
                break;
            case APT.BarButtonSystemItem.Rewind:
                this.setTitle("Rewind");
                break;
            case APT.BarButtonSystemItem.FastForward:
                this.setTitle("FastForward");
                break;
            case APT.BarButtonSystemItem.Undo:
                this.setTitle("Undo");
                break;
            case APT.BarButtonSystemItem.Redo:
                this.setTitle("Redo");
                break;
            case APT.BarButtonSystemItem.PageCurl:
                this.setTitle("Next Page");
                break;
            default:
                this._style = 0;
                break;
        }
    },

    triggerTargetAction: function(event) {
        if (this._target !== null && this._action !== null && this._action !== "")
        {
            (this._target[this._action.valueOf()])(event);
        }
    },

});


