/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.Calendar = Class.$define("APT.Calendar", {
    __init__: function(calendarIdentifier) {
        this._calendarIdentifier = null;
        this._firstWeekDay = null;
        if (arguments.length > 0)
        {
            this._calendarIdentifier = calendarIdentifier;
            this._firstWeekDay = 1;
        }
    },

    $copy: function() {
        var copyObject = new APT.Calendar();
        copyObject._calendarIdentifier = this._calendarIdentifier;
        copyObject._firstWeekDay = this._firstWeekDay;
        return copyObject;
    },

    numberOfWeeks: function(weeks, date, result) {
        var fin = date.getTime() - result.getTime();
        var dias = Math.floor(fin / (1000 * 60 * 60 * 24));
        if (dias > 0)
        {
            weeks = weeks + (Math.ceil(dias / 7) - 1);
        }

        return weeks;
    },

    getWeekNumber: function(date, dateComponent) {
        var date_ = new Date(date.getFullYear(), date.getMonth(), date.getDate());

        var oneJanuaryCurrentYear = new Date(date.getFullYear(), 0, 1);
        var week = 0;
        var weekNumber = 0;
        var occurence = 1;
        var co = 1;
        var weekday = this._firstWeekDay;
        var result;

        while (occurence > 0)
        {
            result = new Date(date.getFullYear(), 0, co);
            result.getDay() === weekday && occurence--;
            co++;
        }

        if (date_.getTime() < result.getTime() && date_.getTime() >= new Date(result.getTime() + -7 * 24 * 3600 * 1000).getTime() || result.getDate() === 1 && result.getMonth() === 0)
        {
            week = 1;
        }
        else
        {
            if (date_.getTime() >= new Date(date.getFullYear(), 11, 25).getTime() && date_.getTime() <= new Date(date.getFullYear(), 11, 31).getTime())
            {
                var d = new Date(date.getFullYear(), 11, 31);
                while (d.getDay() !== weekday)
                {
                    d = new Date(d.getTime() + -1 * 24 * 3600 * 1000);
                }

                if (date_.getTime() >= d.getTime())
                {
                    week = 1;
                }
                else
                {
                    weekNumber = 2;
                    week = this.numberOfWeeks(weekNumber, date, result);
                }
            }
            else
            {
                weekNumber = 2;
                week = this.numberOfWeeks(weekNumber, date, result);
            }
        }

        return week;
    },

    calendarIdentifier: function() {
        return this._calendarIdentifier;
    },

    components: function(unitFlags, date) {
        var dateComponent = new APT.DateComponents();
        dateComponent.setUnitFlags(unitFlags);

        if (APT.CalendarUnit.Era & unitFlags)
        {
            var era = (((date.getFullYear() < 0)) ? (0) : (1));
            dateComponent.setEra(era);
        }

        if (APT.CalendarUnit.Year & unitFlags)
        {
            dateComponent.setYear(date.getFullYear());
        }

        if (APT.CalendarUnit.Month & unitFlags)
        {
            dateComponent.setMonth(date.getMonth() + 1);
        }

        if (APT.CalendarUnit.Day & unitFlags)
        {
            dateComponent.setDay(date.getDate());
        }

        if (APT.CalendarUnit.Hour & unitFlags)
        {
            dateComponent.setHour(date.getHours());
        }

        if (APT.CalendarUnit.Minute & unitFlags)
        {
            dateComponent.setMinute(date.getMinutes());
        }

        if (APT.CalendarUnit.Second & unitFlags)
        {
            dateComponent.setSecond(date.getSeconds());
        }

        var calendar = new APT.Calendar(APT.CalendarKeys.Gregorian);
        dateComponent.setCalendar(calendar, date);

        if (APT.CalendarUnit.Weekday & unitFlags)
        {
            dateComponent.setWeekday(date.getDay() + 1);
        }

        if (APT.CalendarUnit.Week & unitFlags)
        {
            var week = this.getWeekNumber(date, dateComponent);
            dateComponent.setWeek(week);
        }

        if (APT.CalendarUnit.WeekdayOrdinal & unitFlags)
        {
            var currentMonth = date.getMonth() + 1;
            var otherDate = new Date(date.getTime() + -7 * 24 * 3600 * 1000);
            var i = 0;
            for (i = 1; currentMonth === otherDate.getMonth() + 1; i++)
            {
                otherDate = new Date(otherDate.getTime() + -7 * 24 * 3600 * 1000);
            }

            dateComponent.setWeekdayOrdinal(i);
        }

        return dateComponent;
    },

});


