/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.Color = Class.$define("APT.Color", {
    __init__: function() {
        this._red = 0;
        this._green = 0;
        this._blue = 0;
        this._alpha = 1.0;
        this._type = 0;
        this._type = APT.Color.RGBA;
    },

    $copy: function() {
        var copyObject = new APT.Color();
        copyObject._red = this._red;
        copyObject._green = this._green;
        copyObject._blue = this._blue;
        copyObject._alpha = this._alpha;
        copyObject._type = this._type;
        return copyObject;
    },

    setComponents: function(rgbaComponents, aType, fromCss) {
        if (rgbaComponents !== null && rgbaComponents.length === 4)
        {
            if (fromCss)
            {
                this._red = this.normalize(parseFloat(rgbaComponents[0]));
                this._green = this.normalize(parseFloat(rgbaComponents[1]));
                this._blue = this.normalize(parseFloat(rgbaComponents[2]));
            }
            else
            {
                this._red = parseInt(this.normalize(parseFloat(rgbaComponents[0]) * 255));
                this._green = parseInt(this.normalize(parseFloat(rgbaComponents[1]) * 255));
                this._blue = parseInt(this.normalize(parseFloat(rgbaComponents[2]) * 255));
            }

            this._alpha = this.normalizeAlpha(parseFloat(rgbaComponents[3]));
            this._type = aType;
        }
    },

    normalize: function(aValue) {
        if (aValue < 0.0)
        {
            return 0;
        }
        if (aValue > 255.0)
        {
            return 255;
        }
        return aValue;
    },

    normalizeAlpha: function(aValue) {
        if (aValue < 0.0)
        {
            return 0;
        }
        if (aValue > 1.0)
        {
            return 1;
        }
        return aValue;
    },

    getHtmlValue: function() {
        switch(this._type)
        {
            case APT.Color.RGBA:
            case APT.Color.WHITE_ALPHA:
                var value = "rgba(";
                value += "" + Math.floor(this._red);
                value += "," + Math.floor(this._green);
                value += "," + Math.floor(this._blue);
                value += "," + this._alpha + ")";
                return value;
                break;
            default:
                break;
        }

        return null;
    },

    getRed: function() {
        return Math.round(this._red / 255 * 100) / 100;
    },

    getGreen: function() {
        return Math.round(this._green / 255 * 100) / 100;
    },

    getBlue: function() {
        return Math.round(this._blue / 255 * 100) / 100;
    },

    getAlpha: function() {
        return this._alpha;
    },

    getWhite: function() {
        return Math.round(this._red / 255 * 100) / 100;
    },

    isValidWhiteAlpha: function() {
        return this._type === APT.Color.WHITE_ALPHA;
    },

    getWhite_alpha: function(white, alpha) {
        return this.isValidWhiteAlpha();
    },

    getColorType: function() {
        return this._type;
    },

    parseRgbValues: function(cssStringColor) {
        if (cssStringColor.valueOf() === "transparent")
        {
            cssStringColor = "rgba(0, 0, 0, 0)";
        }

        var cssColorType = cssStringColor.substring(0, 4);
        var auxStr = null;
        var arrayRgb = new Array();
        if (cssColorType === "rgb(")
        {
            auxStr = cssStringColor.substring(4, cssStringColor.length - 1);
            arrayRgb[0] = auxStr.substring(0, auxStr.indexOf(","));

            auxStr = auxStr.substring((arrayRgb[0]).length + 1);
            arrayRgb[1] = auxStr.substring(0, auxStr.indexOf(","));

            auxStr = auxStr.substring((arrayRgb[1]).length + 1);
            arrayRgb[2] = auxStr.trim();
        }
        else
        {
            if (cssColorType === "rgba")
            {
                auxStr = cssStringColor.substring(5, cssStringColor.length - 1);
                arrayRgb[0] = auxStr.substring(0, auxStr.indexOf(","));

                auxStr = auxStr.substring((arrayRgb[0]).length + 1);
                arrayRgb[1] = auxStr.substring(0, auxStr.indexOf(","));

                auxStr = auxStr.substring((arrayRgb[1]).length + 1);
                arrayRgb[2] = auxStr.substring(0, auxStr.indexOf(","));

                auxStr = auxStr.substring((arrayRgb[2]).length + 1);
                arrayRgb[3] = auxStr.trim();
            }
        }

        for (var i = 0; i < arrayRgb.length; i++)
        {
            arrayRgb[i] = (arrayRgb[i]).trim();
        }

        return arrayRgb;
    },

    updateFromCSS: function(cssStringColor) {
        if (cssStringColor !== this.getHtmlValue())
        {
            var rgbArray = this.parseRgbValues(cssStringColor);

            if (rgbArray.length === 4)
            {
                this.setComponents(rgbArray, this.getColorType(), true);
            }
            else
            {
                if (rgbArray.length === 3)
                {
                    var auxArray = new Array();
                    auxArray[0] = rgbArray[0];
                    auxArray[1] = rgbArray[1];
                    auxArray[2] = rgbArray[2];
                    auxArray[3] = 1;
                    this.setComponents(auxArray, this.getColorType(), true);
                }
            }
        }

        return this;
    },

    isEqual: function(aColor) {
        if (aColor !== null && aColor !== undefined && aColor instanceof APT.Color)
        {
            if (this === aColor)
            {
                return true;
            }
            else
            {
                return this._type === aColor._type && this.getRed() === aColor.getRed() && this.getGreen() === aColor.getGreen() && this.getBlue() === aColor.getBlue() && this.getAlpha() === aColor.getAlpha();
            }
        }

        return false;
    },

});

APT.Color.RGBA = 0;

APT.Color.WHITE_ALPHA = 1;

APT.Color.New = function(rgbaComponents) {
    var color = new APT.Color();
    color.setComponents(rgbaComponents, APT.Color.RGBA, false);
    return color;
};

APT.Color.RGBAColor = function(red, green, blue, alpha) {
    var color = new APT.Color();
    var rgbaComponents = [red, green, blue, alpha];
    color.setComponents(rgbaComponents, APT.Color.RGBA, false);
    return color;
};

APT.Color.colorWithWhite = function(white, alpha) {
    var color = new APT.Color();
    var components = [white, white, white, alpha];
    color.setComponents(components, APT.Color.WHITE_ALPHA, false);
    return color;
};

APT.Color.validate = function(aValue) {
    if (0.0 <= aValue && aValue <= 255.0)
    {
        return true;
    }
    else
    {
        return false;
    }
};

APT.Color.Black = APT.Color.RGBAColor(0, 0, 0, 1);

APT.Color.White = APT.Color.colorWithWhite(1, 1);

APT.Color.LightGray = APT.Color.colorWithWhite(2 / 3, 1);

APT.Color.Blue = APT.Color.RGBAColor(0, 0, 1, 1);

APT.Color.Green = APT.Color.RGBAColor(0, 1, 0, 1);

APT.Color.Red = APT.Color.RGBAColor(1, 0, 0, 1);

APT.Color.DarkGray = APT.Color.colorWithWhite(1 / 3, 1);

APT.Color.Gray = APT.Color.colorWithWhite(1 / 2, 1);

APT.Color.Cyan = APT.Color.RGBAColor(0, 1, 1, 1);

APT.Color.Yellow = APT.Color.RGBAColor(1, 1, 0, 1);

APT.Color.Magenta = APT.Color.RGBAColor(1, 0, 1, 1);

APT.Color.Orange = APT.Color.RGBAColor(1, 1 / 2, 0, 1);

APT.Color.Purple = APT.Color.RGBAColor(1 / 2, 0, 1 / 2, 1);

APT.Color.Brown = APT.Color.RGBAColor(6 / 10, 4 / 10, 2 / 10, 1);

APT.Color.Transparent = APT.Color.RGBAColor(0, 0, 0, 0);

APT.Color.isValidRed_green_blue_alpha = function(red, green, blue, alpha) {
    if (APT.Color.validate(red) && APT.Color.validate(green) && APT.Color.validate(blue) && APT.Color.validate(alpha))
    {
        return true;
    }
    else
    {
        return false;
    }
};


APT.ColorSpace = Class.$define("APT.ColorSpace", {
    __init__: function() {
    },

    createColor: function(components) {
        return APT.Color.New(components);
    },

});


