/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.Dictionary = Class.$define("APT.Dictionary", {
    __init__: function() {
    },

    initWithDictionary: function(otherDictionary) {
        if (otherDictionary !== null)
        {
            var keys = Object.keys(otherDictionary);

            for (var i = 0; i < keys.length; i++)
            {
                this[keys[i]] = otherDictionary[keys[i]];
            }
        }

        return this;
    },

    objectFor: function(key) {
        if (this[key] === undefined)
        {
            return null;
        }

        return this[key];
    },

    getEnumerator: function() {
        var keys = Object.keys(this);
        var values = new Array();

        for (var i = 0; i < keys.length; i++)
        {
            values.push(this[keys[i]]);
        }

        return APT.Enumerator.New(values);
    },

    allValues: function() {
        var keys = Object.keys(this);
        var values = new Array();

        for (var i = 0; i < keys.length; i++)
        {
            values.push(this[keys[i]]);
        }

        return values;
    },

    clear: function() {
        var keys = Object.keys(this);
        for (var i = 0; i < keys.length; i++)
        {
            delete this[keys[i]];
        }
    },

    setValue_ForKey: function(value, key) {
        if (value === undefined || value === null)
        {
            delete this[key];
        }
        else
        {
            this[key] = value;
        }
    },

    removeObjectForKey: function(key) {
        delete this[key];
    },

    removeObjectsForKeys: function(keys) {
        for (var i = 0; i < keys.length; i++)
        {
            this.removeObjectForKey(keys[i]);
        }
    },

    allKeysForObject: function(obj) {
        var resultKeys = new Array();
        var allKeys = Object.keys(this);
        for (var i = 0; i < allKeys.length; i++)
        {
            if (APT.Global.isEqual(this[allKeys[i]], obj))
            {
                resultKeys.push(allKeys[i]);
            }
        }

        return resultKeys;
    },

    setDictionary: function(otherDictionary) {
        this.clear();

        if (otherDictionary !== null)
        {
            var keys = Object.keys(otherDictionary);

            for (var i = 0; i < keys.length; i++)
            {
                this[keys[i]] = otherDictionary[keys[i]];
            }
        }
    },

    addEntriesFromDictionary: function(otherDictionary) {
        if (otherDictionary !== null)
        {
            var keys = Object.keys(otherDictionary);

            for (var i = 0; i < keys.length; i++)
            {
                this[keys[i]] = otherDictionary[keys[i]];
            }
        }
    },

    isEqual: function(obj) {
        if (obj !== null && obj !== undefined && obj instanceof APT.Dictionary)
        {
            if (this === obj)
            {
                return true;
            }
            else
            {
                return this.allValues().apt_isEqual((obj).allValues()) && Object.keys(this).apt_isEqual(Object.keys(obj));
            }
        }

        return false;
    },

});

APT.Dictionary.dictionaryWithDictionary = function(otherDictionary) {
    var dictionary = new APT.Dictionary();
    if (otherDictionary !== null)
    {
        var keys = Object.keys(otherDictionary);

        for (var i = 0; i < keys.length; i++)
        {
            dictionary[keys[i]] = otherDictionary[keys[i]];
        }
    }

    return dictionary;
};

APT.Dictionary.dictionaryWithObjectsForKeys = function(objects, keys) {
    var dictionary = new APT.Dictionary();
    for (var i = 0; i < objects.length; i++)
    {
        dictionary[keys[i]] = objects[i];
    }

    return dictionary;
};

APT.Dictionary.dictionaryWithObjectsForKeysCount = function(objects, keys, count) {
    var dictionary = new APT.Dictionary();
    for (var i = 0; i < count; i++)
    {
        dictionary[keys[i]] = objects[i];
    }

    return dictionary;
};


