/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.Font = Class.$define("APT.Font", {
    __init__: function() {
        this._pointSize = 0;
        this._familyName = null;
        this._fontName = null;
        this._style = null;
        this._fontWeight = null;
        this._fontName = "Helvetica";
        this._familyName = "Helvetica";
        this._style = APT.Font.NORMAL;
        this._fontWeight = APT.Font.NORMAL;
        this._pointSize = 12;
    },

    $copy: function() {
        var copyObject = new APT.Font();
        copyObject._pointSize = this._pointSize;
        copyObject._familyName = this._familyName;
        copyObject._fontName = this._fontName;
        copyObject._style = this._style;
        copyObject._fontWeight = this._fontWeight;
        return copyObject;
    },

    fontWithSize: function(fontSize) {
        var font = new APT.Font();
        font._fontName = this._fontName;
        font._familyName = this._familyName;
        font._style = this._style;
        font._fontWeight = this._fontWeight;
        font._pointSize = fontSize;
        return font;
    },

    size: function() {
        return this._pointSize;
    },

    setSize: function(size) {
        this._pointSize = size;
    },

    familyName: function() {
        return this._familyName;
    },

    setFamilyName: function(family) {
        this._familyName = family;
    },

    fontName: function() {
        return this._fontName;
    },

    setFontName: function(fontName) {
        this._fontName = fontName;
    },

    style: function() {
        return this._style;
    },

    setStyle: function(style) {
        this._style = style;
    },

    fontWeight: function() {
        return this._fontWeight;
    },

    setFontWeight: function(fontWeight) {
        this._fontWeight = fontWeight;
    },

    updateFromCSS: function(jqobject) {
        this.setSize(parseFloat(jqobject.css("font-size")));
        this.setFamilyName(jqobject.css("font-family"));
        this.setFontWeight(jqobject.css("font-weight"));
        this.setStyle(jqobject.css("font-style"));

        if (jqobject.css("font-weight") === "bold" || jqobject.css("font-weight") === "700")
        {
            if (jqobject[0].style.fontStyle === "italic")
            {
                this.setFontName(jqobject.css("font-family") + "-" + APT.Font.BOLD_ITALIC);
            }
            else
            {
                if (jqobject[0].style.fontStyle === "oblique")
                {
                    this.setFontName(jqobject.css("font-family") + "-" + APT.Font.BOLD_OBLIQUE);
                }
                else
                {
                    this.setFontName(jqobject.css("font-family") + "-" + APT.Font.BOLD);
                }
            }
        }
        else
        {
            if (jqobject[0].style.fontStyle === "oblique")
            {
                this.setFontName(jqobject.css("font-family") + "-" + APT.Font.OBLIQUE);
            }
            else
            {
                if (jqobject[0].style.fontStyle === "italic")
                {
                    this.setFontName(jqobject.css("font-family") + "-" + APT.Font.ITALIC);
                }
            }
        }

        return this;
    },

    updateCSS: function(jqobject) {
        jqobject.css("font-size", this.size());
        jqobject.css("font-family", this.familyName());
        jqobject.css("font-style", this.style());
        jqobject.css("font-weight", this.fontWeight());
    },

});

APT.Font.OBLIQUE = "Oblique";

APT.Font.BOLD = "Bold";

APT.Font.BOLD_OBLIQUE = "BoldOblique";

APT.Font.ITALIC = "Italic";

APT.Font.BOLD_ITALIC = "BoldItalic";

APT.Font.NORMAL = "normal";

APT.Font.fontWithNameSize = function(familyName, fontSize) {
    var font = new APT.Font();
    font._fontName = familyName;
    var index = familyName.indexOf("-");

    if (index > -1)
    {
        font._familyName = familyName.slice(0, index);
        var name = familyName.slice(index + 1);

        switch(name.valueOf())
        {
            case APT.Font.OBLIQUE.valueOf():
                font._style = "oblique";
                break;
            case APT.Font.BOLD.valueOf():
                font._fontWeight = "bold";
                break;
            case APT.Font.BOLD_OBLIQUE.valueOf():
                font._fontWeight = "bold";
                font._style = "oblique";
                break;
            case APT.Font.ITALIC.valueOf():
                font._style = "italic";
                break;
            case APT.Font.BOLD_ITALIC.valueOf():
                font._style = "italic";
                font._fontWeight = "bold";
                break;
        }
    }
    else
    {
        font._familyName = familyName;
    }

    font._pointSize = fontSize;
    return font;
};

APT.Font.systemFontOfSize = function(fontSize) {
    var font = new APT.Font();
    font._pointSize = fontSize;
    return font;
};

APT.Font.boldSystemFontOfSize = function(fontSize) {
    var font = new APT.Font();
    font._pointSize = fontSize;
    font._fontName = "Helvetica-Bold";
    font._fontWeight = "bold";
    return font;
};

APT.Font.italicSystemFontOfSize = function(fontSize) {
    var font = new APT.Font();
    font._fontName = "Helvetica-Oblique";
    font._pointSize = fontSize;
    font._style = "oblique";
    return font;
};


