/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

// Extension to class "APT.Global"
// To extend a class add methods to .prototype property
APT.Global.THROW_IF_NOT_IMPLEMENTED = true;

APT.Global.NotFound = -1;

APT.Global.RAND_MAX = 4294967296;

APT.Global.createUserObject = function(className) {
    var constructor = window["application"][className];
    var obj = Object.create(constructor.prototype);
    constructor.apply(obj);
    return obj;
};

APT.Global.processElement = function(dom, container) {
    var subview;

    dom.each(function(index, element) {
        var className = element.attributes["data-apt-class"];

        if (className !== undefined)
        {
            subview = APT.Global.createAPTObject(className.value);

            var markup = subview.getRenderMarkup();

            for (var i = 0; i < element.attributes.length; i++)
            {
                if (element.attributes[i].name !== "type")
                {
                    markup = markup.attr(element.attributes[i].name, element.attributes[i].nodeValue);
                }
            }

            subview.processNode(markup, jQuery(element).clone());

            if (container !== undefined)
            {
                APT.Global.processSubview(container, subview, className.value);
            }
        }
    });
    return subview;
};

APT.Global.processSubview = function(container, subview, className) {
    if (container["contentView"] instanceof Function)
    {
        if ((container).contentView() === null)
        {
            (container).setContentView(subview);
            (container).contentView().setBackgroundColor(APT.Color.Transparent);
            (container).addSubview(subview);
        }
    }
    else
    {
        if (container instanceof APT.NavigationBar && className !== "NavigationBar")
        {
            (container).pushNavigationItem_animated(subview, false);
        }
        else
        {
            if (container instanceof APT.TableView)
            {
                (container).setTableHeaderView(subview);
            }
            else
            {
                if (container instanceof Array)
                {
                    if (subview instanceof APT.TabBarButtonDefault)
                    {
                        var tabBarItem = new APT.TabBarItem();
                        tabBarItem.setTabBarButton(subview);
                        var imageName = (subview).jQElement().find("a").attr("data-apt-image");
                        var image = null;
                        if (imageName !== undefined && imageName.length > 0)
                        {
                            image = APT.Image.imageNamed(imageName);
                        }

                        tabBarItem.defaultView().initWithTitle_image_tag((subview).jQElement().text().trim(), image, 0);
                        (container).push(tabBarItem);
                    }
                    else
                    {
                        if (subview instanceof APT.BarButtonDefault)
                        {
                            var item = new APT.BarButtonItem();
                            item.setDefaultView(subview);
                            item.defaultView()._barButtonItem = item;
                            item.defaultView().addTarget_action_forControlEvents(item, "triggerTargetAction", APT.ControlEvent.vmouseup);
                            item.setTitle((subview).jQElement().text());
                            (container).push(item);
                        }
                    }
                }
                else
                {
                    (container).addSubviewFromXibFiles(subview);
                }
            }
        }
    }
};

APT.Global.setNeedsLayout = function(view) {
    if (view === undefined || view === null)
    {
        return;
    }
    view.setNeedsLayout();
    for (var i = 0; i < view.subviews().length; i++)
    {
        if (view.subviews()[i] !== undefined)
        {
            APT.Global.setNeedsLayout(view.subviews()[i]);
        }
    }
};

APT.Global.isMemberOfClass = function(obj1, classObj) {
    if (obj1["$class"] !== undefined)
    {
        return obj1["$class"] === classObj;
    }
    else
    {
        return false;
    }
};

APT.Global.conformsToProtocol = function(obj, protocolName) {
    return true;
};

APT.Global.isEqual = function(obj1, obj2) {
    if (obj1 !== null && obj1 !== undefined)
    {
        if (obj1["isEqual"] instanceof Function)
        {
            return (obj1["isEqual"])(obj2);
        }
        else
        {
            if (obj1["apt_isEqual"] instanceof Function)
            {
                return (obj1["apt_isEqual"])(obj2);
            }
            else
            {
                if (obj2 !== null && obj2 !== undefined && (obj1).href && (obj2).href)
                {
                    return (obj1).href.valueOf() === (obj2).href.valueOf();
                }
                else
                {
                    return obj1 === obj2;
                }
            }
        }
    }
    else
    {
        return false;
    }
};

APT.Global.hash = function(obj1) {
    if (obj1["hash"] instanceof Function)
    {
        return (obj1["hash"])();
    }
    else
    {
        return obj1;
    }
};

APT.Global.truncate = function(number) {
    return (((number < 0)) ? (Math.ceil(number)) : (Math.floor(number)));
};


APT.StringEncoding = Class.$define("APT.StringEncoding", {
    __init__: function() {
    },

});

APT.StringEncoding.ASCII = 0;


APT.StringOptions = Class.$define("APT.StringOptions", {
    __init__: function() {
    },

});

APT.StringOptions.CaseInsensitiveSearch = 2;

APT.StringOptions.LiteralSearch = 4;

APT.StringOptions.BackwardsSearch = 8;

APT.StringOptions.AnchoredSearch = 16;

APT.StringOptions.NumericSearch = 64;


APT.RectEdge = Class.$define("APT.RectEdge", {
    __init__: function() {
    },

});

APT.RectEdge.MinX = 0;

APT.RectEdge.MinY = 1;

APT.RectEdge.MaxX = 2;

APT.RectEdge.MaxY = 3;


APT.ComparisonResult = Class.$define("APT.ComparisonResult", {
    __init__: function() {
    },

});

APT.ComparisonResult.OrderedAscending = -1;

APT.ComparisonResult.OrderedSame = 0;

APT.ComparisonResult.OrderedDescending = 1;


APT.TableViewStyle = Class.$define("APT.TableViewStyle", {
    __init__: function() {
    },

});

APT.TableViewStyle.Plain = 0;

APT.TableViewStyle.Grouped = 1;


APT.TableViewRowAnimation = Class.$define("APT.TableViewRowAnimation", {
    __init__: function() {
    },

});

APT.TableViewRowAnimation.None = 0;


APT.TableViewCellStyle = Class.$define("APT.TableViewCellStyle", {
    __init__: function() {
    },

});

APT.TableViewCellStyle.Default = 0;

APT.TableViewCellStyle.Value1 = 1;

APT.TableViewCellStyle.Value2 = 2;

APT.TableViewCellStyle.Subtitle = 3;


APT.TableViewCellAccessoryType = Class.$define("APT.TableViewCellAccessoryType", {
    __init__: function() {
    },

});

APT.TableViewCellAccessoryType.None = 0;

APT.TableViewCellAccessoryType.DisclosureIndicator = 1;

APT.TableViewCellAccessoryType.Checkmark = 2;


APT.TextFieldBorderStyle = Class.$define("APT.TextFieldBorderStyle", {
    __init__: function() {
    },

});

APT.TextFieldBorderStyle.None = 0;

APT.TextFieldBorderStyle.Line = 1;

APT.TextFieldBorderStyle.Bezel = 2;

APT.TextFieldBorderStyle.RoundedRect = 3;


APT.AnimationTransition = Class.$define("APT.AnimationTransition", {
    __init__: function() {
    },

});

APT.AnimationTransition.None = 0;

APT.AnimationTransition.FlipFromLeft = 1;

APT.AnimationTransition.FlipFromRight = 2;


APT.FormatterDate = Class.$define("APT.FormatterDate", {
    __init__: function() {
    },

});

APT.FormatterDate.NoStyle = 0;

APT.FormatterDate.ShortStyle = 1;

APT.FormatterDate.MediumStyle = 2;

APT.FormatterDate.LongStyle = 3;


APT.KeyboardType = Class.$define("APT.KeyboardType", {
    __init__: function() {
    },

});

APT.KeyboardType.Default = 0;

APT.KeyboardType.PhonePad = 1;

APT.KeyboardType.NumberPad = 2;

APT.KeyboardType.URL = 3;

APT.KeyboardType.EmailAddress = 4;


APT.DatePickerMode = Class.$define("APT.DatePickerMode", {
    __init__: function() {
    },

});

APT.DatePickerMode.Time = 0;

APT.DatePickerMode.Date = 1;

APT.DatePickerMode.DateAndTime = 2;

APT.DatePickerMode.CountDownTimer = 3;


APT.PopoverArrowDirection = Class.$define("APT.PopoverArrowDirection", {
    __init__: function() {
    },

});

APT.PopoverArrowDirection.Up = 0;


APT.ViewAnimationCurve = Class.$define("APT.ViewAnimationCurve", {
    __init__: function() {
    },

});

APT.ViewAnimationCurve.EaseInOut = "easeInOutQuad";

APT.ViewAnimationCurve.EaseIn = "easeInQuad";

APT.ViewAnimationCurve.EaseOut = "easeOutQuad";

APT.ViewAnimationCurve.Linear = "linear";


APT.TextAlignment = Class.$define("APT.TextAlignment", {
    __init__: function() {
    },

});

APT.TextAlignment.Left = 0;

APT.TextAlignment.Center = 1;

APT.TextAlignment.Right = 2;

APT.TextAlignment.toCSSTextAlign = function(aTextAlignment) {
    switch(aTextAlignment)
    {

        case APT.TextAlignment.Left:
            return "left";
        case APT.TextAlignment.Center:
            return "center";
        case APT.TextAlignment.Right:
            return "right";
        default:
            return "left";
    }
};

APT.TextAlignment.parseCSSTextAlign = function(aCSSTextAlignment) {
    if (aCSSTextAlignment === undefined || aCSSTextAlignment === null)
    {
        return APT.TextAlignment.Left;
    }
    else
    {
        switch(aCSSTextAlignment.valueOf())
        {

            case "left":
                return APT.TextAlignment.Left;
            case "center":
                return APT.TextAlignment.Center;
            case "right":
                return APT.TextAlignment.Right;
            default:
                return APT.TextAlignment.Left;
        }
    }
};


APT.ButtonType = Class.$define("APT.ButtonType", {
    __init__: function() {
    },

});

APT.ButtonType.Custom = 0;

APT.ButtonType.RoundedRect = 1;

APT.ButtonType.DetailDisclosure = 2;

APT.ButtonType.InfoLight = 3;

APT.ButtonType.InfoDark = 4;

APT.ButtonType.Add = 5;


APT.LineBreakMode = Class.$define("APT.LineBreakMode", {
    __init__: function() {
    },

});

APT.LineBreakMode.BreakWord = 0;

APT.LineBreakMode.Normal = 2;

APT.LineBreakMode.toCSSWordWrap = function(aLineBreakMode) {
    switch(aLineBreakMode)
    {

        case APT.LineBreakMode.Normal:
            return "normal";
            break;
        case APT.LineBreakMode.BreakWord:
            return "break-word";
            break;
        default:
            return "break-word";
            break;
    }
};

APT.LineBreakMode.parseLineBreakMode = function(aCSSWordWrap) {
    if (aCSSWordWrap === undefined || aCSSWordWrap === null)
    {
        return APT.LineBreakMode.BreakWord;
    }
    else
    {
        switch(aCSSWordWrap.valueOf())
        {

            case "normal":
                return APT.LineBreakMode.Normal;
                break;
            case "break-word":
                return APT.LineBreakMode.BreakWord;
                break;
            default:
                return APT.LineBreakMode.BreakWord;
                break;
        }
    }
};


APT.ImageOrientation = Class.$define("APT.ImageOrientation", {
    __init__: function() {
    },

});

APT.ImageOrientation.Up = 0;

APT.ImageOrientation.Down = 1;

APT.ImageOrientation.Left = 2;

APT.ImageOrientation.Right = 3;


APT.ProgressViewStyle = Class.$define("APT.ProgressViewStyle", {
    __init__: function() {
    },

});

APT.ProgressViewStyle.Default = 0;

APT.ProgressViewStyle.Bar = 1;


APT.DeviceOrientation = Class.$define("APT.DeviceOrientation", {
    __init__: function() {
    },

});

APT.DeviceOrientation.Unknown = 0;

APT.DeviceOrientation.Portrait = 1;

APT.DeviceOrientation.Landscape = 2;

APT.DeviceOrientation.FaceUp = 3;


APT.InterfaceOrientation = Class.$define("APT.InterfaceOrientation", {
    __init__: function() {
    },

});

APT.InterfaceOrientation.Unknown = 0;

APT.InterfaceOrientation.Portrait = 1;

APT.InterfaceOrientation.Landscape = 2;


APT.UserInterfaceIdiom = Class.$define("APT.UserInterfaceIdiom", {
    __init__: function() {
    },

});

APT.UserInterfaceIdiom.Phone = 0;

APT.UserInterfaceIdiom.Pad = 1;


APT.ActivityIndicatorViewStyle = Class.$define("APT.ActivityIndicatorViewStyle", {
    __init__: function() {
    },

});

APT.ActivityIndicatorViewStyle.WhiteLarge = 0;

APT.ActivityIndicatorViewStyle.White = 1;

APT.ActivityIndicatorViewStyle.Gray = 2;


APT.NSDataSearch = Class.$define("APT.NSDataSearch", {
    __init__: function() {
    },

});

APT.NSDataSearch.Backwards = 0;

APT.NSDataSearch.Anchored = 1;


APT.ControlState = Class.$define("APT.ControlState", {
    __init__: function() {
    },

});

APT.ControlState.Normal = 0;

APT.ControlState.Highlighted = 1;

APT.ControlState.Disabled = 2;

APT.ControlState.Selected = 4;

APT.ControlState.Application = 0x00FF0000;

APT.ControlState.Reserved = 0xFF000000;


APT.ControlEvent = Class.$define("APT.ControlEvent", {
    __init__: function() {
    },

});

APT.ControlEvent.vmousedown = 1 << 0;

APT.ControlEvent.TouchDownRepeat = 1 << 1;

APT.ControlEvent.vmouseover = 1 << 2;

APT.ControlEvent.TouchDragOutside = 1 << 3;

APT.ControlEvent.TouchDragEnter = 1 << 4;

APT.ControlEvent.TouchDragExit = 1 << 5;

APT.ControlEvent.vmouseup = 1 << 6;

APT.ControlEvent.TouchUpOutside = 1 << 7;

APT.ControlEvent.vmousecancel = 1 << 8;

APT.ControlEvent.ValueChanged = 1 << 12;

APT.ControlEvent.focus = 1 << 16;

APT.ControlEvent.EditingChanged = 1 << 17;

APT.ControlEvent.change = 1 << 18;

APT.ControlEvent.blur = 1 << 19;

APT.ControlEvent.AllTouchEvents = 0x00000FFF;

APT.ControlEvent.AllEditingEvents = 0x000F0000;

APT.ControlEvent.ApplicationReserved = 0x0F000000;

APT.ControlEvent.SystemReserved = 0xF0000000;

APT.ControlEvent.AllEvents = 0xFFFFFFFF;


APT.BarButtonItemStyle = Class.$define("APT.BarButtonItemStyle", {
    __init__: function() {
    },

});

APT.BarButtonItemStyle.Plain = 0;

APT.BarButtonItemStyle.Bordered = 1;

APT.BarButtonItemStyle.Done = 2;


APT.BarStyle = Class.$define("APT.BarStyle", {
    __init__: function() {
    },

});

APT.BarStyle.Default = 0;

APT.BarStyle.Black = 1;


APT.TabBarSystemItem = Class.$define("APT.TabBarSystemItem", {
    __init__: function() {
    },

});

APT.TabBarSystemItem.More = 0;

APT.TabBarSystemItem.Favorites = 1;

APT.TabBarSystemItem.Featured = 2;

APT.TabBarSystemItem.TopRated = 3;

APT.TabBarSystemItem.Recents = 4;

APT.TabBarSystemItem.Contacts = 5;

APT.TabBarSystemItem.History = 6;

APT.TabBarSystemItem.Bookmarks = 7;

APT.TabBarSystemItem.Search = 8;

APT.TabBarSystemItem.Downloads = 9;

APT.TabBarSystemItem.MostRecent = 10;

APT.TabBarSystemItem.MostViewed = 11;


APT.BarButtonSystemItem = Class.$define("APT.BarButtonSystemItem", {
    __init__: function() {
    },

});

APT.BarButtonSystemItem.Done = 0;

APT.BarButtonSystemItem.Cancel = 1;

APT.BarButtonSystemItem.Edit = 2;

APT.BarButtonSystemItem.Save = 3;

APT.BarButtonSystemItem.Add = 4;

APT.BarButtonSystemItem.FlexibleSpace = 5;

APT.BarButtonSystemItem.FixedSpace = 6;

APT.BarButtonSystemItem.Compose = 7;

APT.BarButtonSystemItem.Reply = 8;

APT.BarButtonSystemItem.Action = 9;

APT.BarButtonSystemItem.Organize = 10;

APT.BarButtonSystemItem.Bookmarks = 11;

APT.BarButtonSystemItem.Search = 12;

APT.BarButtonSystemItem.Refresh = 13;

APT.BarButtonSystemItem.Stop = 14;

APT.BarButtonSystemItem.Camera = 15;

APT.BarButtonSystemItem.Trash = 16;

APT.BarButtonSystemItem.Play = 17;

APT.BarButtonSystemItem.Pause = 18;

APT.BarButtonSystemItem.Rewind = 19;

APT.BarButtonSystemItem.FastForward = 20;

APT.BarButtonSystemItem.Undo = 21;

APT.BarButtonSystemItem.Redo = 22;

APT.BarButtonSystemItem.PageCurl = 23;


APT.EventType = Class.$define("APT.EventType", {
    __init__: function() {
    },

});

APT.EventType.Touches = 0;

APT.EventType.Motion = 1;

APT.EventType.RemoteControl = 2;


APT.CalendarKeys = Class.$define("APT.CalendarKeys", {
    __init__: function() {
    },

});

APT.CalendarKeys.Gregorian = new String("gregorian");

APT.CalendarKeys.Buddhist = new String("buddhist");

APT.CalendarKeys.Chinese = new String("chinese");

APT.CalendarKeys.Hebrew = new String("hebrew");

APT.CalendarKeys.Islamic = new String("islamic");

APT.CalendarKeys.IslamicCivil = new String("islamic-civil");

APT.CalendarKeys.Japanese = new String("japanese");

APT.CalendarKeys.RepublicOfChina = new String("roc");

APT.CalendarKeys.Persian = new String("persian");

APT.CalendarKeys.Indian = new String("indian");

APT.CalendarKeys.ISO8601 = new String("");


APT.CalendarUnit = Class.$define("APT.CalendarUnit", {
    __init__: function() {
    },

});

APT.CalendarUnit.Era = 2;

APT.CalendarUnit.Year = 4;

APT.CalendarUnit.Month = 8;

APT.CalendarUnit.Day = 16;

APT.CalendarUnit.Hour = 32;

APT.CalendarUnit.Minute = 64;

APT.CalendarUnit.Second = 128;

APT.CalendarUnit.Week = 256;

APT.CalendarUnit.Weekday = 512;

APT.CalendarUnit.WeekdayOrdinal = 1024;

APT.CalendarUnit.Quarter = 2048;


