/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.IndexPath = Class.$define("APT.IndexPath", {
    __init__: function() {
        this._section = 0;
        this._row = 0;
        this._indexPath = new Array();
    },

    $copy: function() {
        var copyObject = new APT.IndexPath();
        copyObject._section = this._section;
        copyObject._row = this._row;
        copyObject._indexPath = this._indexPath;
        return copyObject;
    },

    initWithIndex: function(anIndex) {
        this._indexPath = [anIndex];
        return this;
    },

    initWithIndexesLength: function(indexes, length) {
        this._indexPath = (indexes).slice(0, length);
        return this;
    },

    indexAtPosition: function(node) {
        return parseInt(this._indexPath[node]);
    },

    indexPathByAddingIndex: function(index) {
        var aIndexPath = new APT.IndexPath();
        aIndexPath._indexPath = this._indexPath.slice();
        aIndexPath._indexPath.push(index);
        return aIndexPath;
    },

    indexPathByRemovingLastIndex: function() {
        var aIndexPath = new APT.IndexPath();
        aIndexPath._indexPath = this._indexPath.slice(0, -1);
        return aIndexPath;
    },

    length: function() {
        return this._indexPath.length;
    },

    row: function() {
        return this._row;
    },

    section: function() {
        return this._section;
    },

    compare: function(anotherPath) {
        var lengthTemp = (((anotherPath.length() < this.length())) ? (anotherPath.length()) : (this.length()));
        var resultTemp = APT.ComparisonResult.OrderedSame;
        for (var i = 0; i < lengthTemp; i++)
        {
            var thisObjectTemp = parseInt(this._indexPath[i]);
            var anotherPathObjectTemp = parseInt(anotherPath._indexPath[i]);
            if (thisObjectTemp > anotherPathObjectTemp)
            {
                resultTemp = APT.ComparisonResult.OrderedDescending;
                break;
            }
            else
            {
                if (thisObjectTemp < anotherPathObjectTemp)
                {
                    resultTemp = APT.ComparisonResult.OrderedAscending;
                    break;
                }
            }
        }

        if (resultTemp === APT.ComparisonResult.OrderedSame)
        {
            if (this.length() < anotherPath.length())
            {
                resultTemp = APT.ComparisonResult.OrderedDescending;
            }
            else
            {
                if (this.length() > anotherPath.length())
                {
                    resultTemp = APT.ComparisonResult.OrderedAscending;
                }
            }
        }

        return resultTemp;
    },

});

APT.IndexPath.indexPathWithIndex = function(index) {
    var aIndexPath = new APT.IndexPath();
    aIndexPath._indexPath = [index];
    return aIndexPath;
};

APT.IndexPath.indexPathWithIndexesLength = function(indexes, length) {
    var aIndexPath = new APT.IndexPath();
    aIndexPath._indexPath = (indexes).slice(0, length);
    return aIndexPath;
};

APT.IndexPath.indexPathForRow_inSection = function(aRow, aSection) {
    var aIndexPath = new APT.IndexPath();
    aIndexPath._section = aSection;
    aIndexPath._row = aRow;
    return aIndexPath;
};


