/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.View = Class.$define("APT.View");

APT.Label = Class.$define("APT.Label", APT.View, {
    __init__: function() {
        this._jqLabel = null;
        this._textColor = APT.Color.Black;
        this._font = new APT.Font();
        this._numberOfLines = 0;
        this.$super();
    },

    $copy: function() {
        var copyObject = new APT.Label();
        copyObject._jqLabel = this._jqLabel;
        copyObject._textColor = this._textColor;
        copyObject._font = this._font;
        copyObject._numberOfLines = this._numberOfLines;
        return copyObject;
    },

    getRenderMarkup: function() {
        return $(APT.Global.toStaticHTML("<div class=\"apt_label_background apt_font\"><div class=\"apt_label\"></div></div>"));
    },

    setJQElement: function(jqelement) {
        var text = jqelement.text();
        this._jqElement = jqelement.trigger("create");
        this._jqLabel = this._jqElement.children(1);

        this.setProperties();

        if (text.length > 0)
        {
            this.setText(text);
        }
        else
        {
            this.setText("");
        }

        this.setNumberOfLines(1);
        this.setLineBreakMode(this.lineBreakMode());
    },

    setText: function(aText) {
        if (aText !== null && aText !== undefined)
        {
            this._jqLabel.text(aText.valueOf());
        }
    },

    text: function() {
        return new String(this._jqLabel.text());
    },

    textAlignment: function() {
        return APT.TextAlignment.parseCSSTextAlign(this._jqLabel.css("text-align"));
    },

    setTextAlignment: function(anAlignment) {
        this._jqLabel.css("text-align", APT.TextAlignment.toCSSTextAlign(anAlignment));
    },

    setEnabled: function(enabled) {
        if (enabled)
        {
            this.jQElement().fadeTo(0.0, 1.0);
        }
        else
        {
            if (this.isEnabled())
            {
                this.jQElement().fadeTo(0.0, 0.5);
            }
        }
    },

    isEnabled: function() {
        return parseFloat(this.jQElement().css("opacity")) > 0.5;
    },

    setTextColor: function(aColor) {
        this._textColor = aColor;
        if (aColor !== null)
        {
            this._jqLabel.css("color", aColor.getHtmlValue().valueOf());
        }
    },

    textColor: function() {
        return this._textColor.updateFromCSS(this._jqLabel.css("color"));
    },

    setNumberOfLines: function(numberOfLines) {
        var labelFontSizeStr = this._jqLabel.css("font-size");
        var labelLineHeightInt = parseInt(parseInt(labelFontSizeStr) * 1.5);
        var unity = labelFontSizeStr.substring(labelFontSizeStr.length - 2);
        if (numberOfLines > 0)
        {
            this._numberOfLines = numberOfLines;
            var labelHeight = this._numberOfLines * labelLineHeightInt;
            this._jqLabel.css("height", new Number(labelHeight).toString() + unity);
            this._jqLabel.css("line-height", new Number(labelLineHeightInt).toString() + unity);
        }
        else
        {
            this._numberOfLines = 0;
            this._jqLabel.css("height", "");
            this._jqLabel.css("line-height", new Number(labelLineHeightInt).toString() + unity);
        }

        this.setNeedsLayout();
    },

    numberOfLines: function() {
        return this._numberOfLines;
    },

    font: function() {
        return this._font.updateFromCSS(this._jqLabel);
    },

    setFont: function(aFont) {
        if (aFont !== null)
        {
            this._font = aFont;
            this._font.updateCSS(this._jqLabel);
        }

        this.setNumberOfLines(this._numberOfLines);
    },

    lineBreakMode: function() {
        return APT.LineBreakMode.parseLineBreakMode(this._jqLabel.css("word-wrap"));
    },

    setLineBreakMode: function(lineBreakMode) {
        this._jqLabel.css("word-wrap", APT.LineBreakMode.toCSSWordWrap(lineBreakMode));
    },

    layoutSubviews: function() {
        var divHeightStr = this.jQElement().css("height");
        var divHeightInt = parseInt(divHeightStr);
        var labelHeightStr = this._jqLabel.css("height");
        var labelHeightInt = parseInt(labelHeightStr);
        var labelMarginTop = (divHeightInt - labelHeightInt) / 2;
        if (labelMarginTop < 0)
        {
            this._jqLabel.css("margin-top", "0px");
        }
        else
        {
            this._jqLabel.css("margin-top", parseInt(labelMarginTop) + "px");
        }
    },

    viewDidAppear: function(animated) {
        this.setNeedsLayout();
        this.$super(animated);
    },

    processNode: function(markup, element) {
        markup.find(".apt_label").text(element.text());

        this.setJQElement(markup);
    },

});


