/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.View = Class.$define("APT.View");

APT.NavigationBar = Class.$define("APT.NavigationBar", APT.View, {
    __init__: function() {
        this._title = null;
        this._prompt = null;
        this._stack = null;
        this._delegate = null;
        this._leftItemsDiv = null;
        this._centerItemsDiv = null;
        this._rightItemsDiv = null;
        this._leftItems = null;
        this._rightItems = null;
        this._backBarButtonItem = null;
        this.$super();
    },

    $copy: function() {
        var copyObject = new APT.NavigationBar();
        copyObject._title = this._title;
        copyObject._prompt = this._prompt;
        copyObject._stack = this._stack;
        copyObject._delegate = this._delegate;
        copyObject._leftItemsDiv = this._leftItemsDiv;
        copyObject._centerItemsDiv = this._centerItemsDiv;
        copyObject._rightItemsDiv = this._rightItemsDiv;
        copyObject._leftItems = this._leftItems;
        copyObject._rightItems = this._rightItems;
        copyObject._backBarButtonItem = this._backBarButtonItem;
        return copyObject;
    },

    getRenderMarkup: function() {
        return $("<div data-role=\"header\" class=\"apt_navbar ui-header ui-bar-b\" data-theme=\"b\"></div>");
    },

    setJQElement: function(jqelement) {
        this._jqElement = jqelement.trigger("create");
        this._stack = APT.Global.CreateStack(null);
    },

    pushNavigationItem_animated: function(aNavigationItem, animated) {
        if (this._leftItemsDiv === null)
        {
            this.setNavigationItem(aNavigationItem);
        }

        var shouldPush = true;
        if (this._delegate !== null && this._delegate["navigationBar_shouldPushItem"] instanceof Function && !this._delegate.navigationBar_shouldPushItem(this, aNavigationItem))
        {
            shouldPush = false;
        }

        if (shouldPush)
        {
            this._stack.push(aNavigationItem);
            aNavigationItem.setNavigationBar(this);
            this.displayItems(animated);

            if (this._delegate !== null && this._delegate["navigationBar_didPushItem"] instanceof Function)
            {
                this._delegate.navigationBar_didPushItem(this, aNavigationItem);
            }
        }
    },

    setNavigationItem: function(aNavigationItem) {
        this._leftItemsDiv = $(aNavigationItem.jQElement().children()[0]).clone();
        this._centerItemsDiv = $(aNavigationItem.jQElement().children()[1]).clone();
        this._rightItemsDiv = $(aNavigationItem.jQElement().children()[2]).clone();
        this._leftItemsDiv.controlgroup();
        this._leftItemsDiv.addClass("style_navigationItem");
        this._rightItemsDiv.controlgroup();
        this._rightItemsDiv.addClass("style_navigationItem");
        this._jqElement.append(this._leftItemsDiv);
        this._jqElement.append(this._centerItemsDiv);
        this._jqElement.append(this._rightItemsDiv);
        this._title = this._centerItemsDiv.find(".apt_navbar_title");
        this._prompt = this._centerItemsDiv.find(".apt_navbar_prompt");
    },

    popNavigationItemAnimated: function(animated) {
        var poppedItem = this._stack.head();
        if (poppedItem === null)
        {
            return null;
        }

        var shouldPop = true;
        if (this._delegate !== null && this._delegate["navigationBar_shouldPopItem"] instanceof Function && !this._delegate.navigationBar_shouldPopItem(this, poppedItem))
        {
            shouldPop = false;
        }

        if (shouldPop)
        {
            this._stack.pop();
            poppedItem.setNavigationBar(null);
            this.displayItems(animated);

            if (this._delegate !== null && this._delegate["navigationBar_didPopItem"] instanceof Function)
            {
                this._delegate.navigationBar_didPopItem(this, poppedItem);
            }
        }

        return poppedItem;
    },

    topItem: function() {
        return this._stack.head();
    },

    backItem: function() {
        return this._stack.second();
    },

    setDelegate: function(aDelegate) {
        this._delegate = aDelegate;
    },

    delegate: function() {
        return this._delegate;
    },

    setItems: function(itemArray) {
        this.setItems_animated(itemArray, false);
    },

    setItems_animated: function(itemArray, animated) {
        if (this._leftItemsDiv === null && itemArray.length > 0)
        {
            this.setNavigationItem(itemArray[0]);
        }

        this._stack = APT.Global.CreateStack(itemArray);
        this.displayItems(animated);
    },

    items: function() {
        return this._stack;
    },

    displayCenterItems: function() {
        this.setupTitle();
    },

    displayLeftItems: function(animated) {
        var topItem = this._stack.head();
        if (topItem.leftBarButtonItems() !== null && topItem.leftBarButtonItems() !== undefined)
        {
            this.setupLeftItems(topItem.leftBarButtonItems(), animated);
        }
    },

    displayRightItems: function(animated) {
        var topItem = this._stack.head();

        if (topItem.rightBarButtonItems() !== null && topItem.rightBarButtonItems() !== undefined)
        {
            this.setupRightItems(topItem.rightBarButtonItems(), animated);
        }
    },

    displayItems: function(animated) {
        if (this._stack.length > 0)
        {
            var topItem = this._stack.head();
            var backItem = this._stack.second();

            this.setupBackButton();
            this.setupTitle();
            this.setupLeftItems(topItem.leftBarButtonItems(), animated);
            this.setupRightItems(topItem.rightBarButtonItems(), animated);
            this.updateButtonsTheme(this.dataTheme());
        }
    },

    setupBackButton: function() {
        if (this._stack.length > 1)
        {
            var topItem = this._stack.head();
            var aBarButtonItem = topItem.backBarButtonItem();
            this._backBarButtonItem = new APT.BackBarButtonDefault();
            if (aBarButtonItem !== null && aBarButtonItem !== undefined && aBarButtonItem.title() !== null && aBarButtonItem.title() !== undefined)
            {
                this._backBarButtonItem.setText(aBarButtonItem.title().valueOf());
                this._backBarButtonItem.addTarget_action_forControlEvents(aBarButtonItem, "triggerTargetAction", APT.ControlEvent.vmouseup);
            }
            else
            {
                this._backBarButtonItem.setText("Back");
                this._backBarButtonItem.addTarget_action_forControlEvents(this, "_apt_NavigationItem_backButtonClicked", APT.ControlEvent.vmouseup);
            }
        }
        else
        {
            this._backBarButtonItem = null;
        }
    },

    barStyle: function() {
        if (this.dataTheme() === "a")
        {
            return APT.BarStyle.Black;
        }

        return APT.BarStyle.Default;
    },

    setBarStyle: function(barStyle) {
        this.setDataTheme(this.parseBarStyleToDataTheme(barStyle));
    },

    dataTheme: function() {
        var dataTheme = this.jQElement().attr("data-theme");
        if (dataTheme === null || dataTheme === undefined)
        {
            dataTheme = "a";
        }

        return dataTheme;
    },

    setDataTheme: function(dataTheme) {
        var oldDataTheme = this.dataTheme();
        if (dataTheme !== null && dataTheme !== undefined && dataTheme.length > 0)
        {
            this.jQElement().attr("data-theme", dataTheme);
            var barClass = "ui-bar-";
            if (this.jQElement().hasClass(barClass + oldDataTheme))
            {
                this.jQElement().removeClass(barClass + oldDataTheme);
            }

            this.jQElement().addClass(barClass + dataTheme);
            this.updateButtonsTheme(dataTheme);
        }
    },

    setupTitle: function() {
        var topItem = this._stack.head();
        this._prompt.children().detach();
        this._title.children().detach();
        if (topItem !== null && topItem !== undefined)
        {
            if (topItem.prompt() !== null && topItem.prompt().trim().length > 0)
            {
                this._prompt.text(topItem.prompt().valueOf());
            }
            else
            {
                this._prompt.text("");
            }

            if (topItem.title() !== null && topItem.title().trim().length > 0)
            {
                this._title.text(topItem.title().valueOf());
            }
            else
            {
                this._title.text("");
            }
        }
        else
        {
            this._prompt.text("");
            this._title.text("");
        }
    },

    setupRightItems: function(rightButtonItems, animated) {
        var thisNavBar = this;

        if (animated)
        {

            this._rightItemsDiv.fadeOut(200, function() {
                thisNavBar.removeRightItems();
                if (rightButtonItems !== null && rightButtonItems !== undefined)
                {
                    thisNavBar.addRightItems(rightButtonItems, animated);
                }
            });
        }
        else
        {
            this._rightItemsDiv.hide();
            this.removeRightItems();
            if (rightButtonItems !== null && rightButtonItems !== undefined)
            {
                thisNavBar.addRightItems(rightButtonItems, animated);
            }
        }
    },

    setupLeftItems: function(leftButtonItems, animated) {
        var thisNavBar = this;
        if (animated)
        {

            this._leftItemsDiv.fadeOut(200, function() {
                thisNavBar.removeLeftItems();
                if (leftButtonItems !== null && leftButtonItems !== undefined)
                {
                    thisNavBar.addLeftItems(leftButtonItems, animated);
                }
            });
        }
        else
        {
            this._leftItemsDiv.hide();
            thisNavBar.removeLeftItems();
            if (leftButtonItems !== null && leftButtonItems !== undefined)
            {
                thisNavBar.addLeftItems(leftButtonItems, animated);
            }
        }
    },

    _apt_NavigationItem_backButtonClicked: function() {
        this.popNavigationItemAnimated(false);
    },

    viewDidAppear: function(animated) {
        this.$super(animated);
        this.displayItems(animated);
    },

    removeLeftItems: function() {
        this._leftItemsDiv.children().detach();
    },

    addBackButtonItem: function(aBarButtonItem) {
        aBarButtonItem.defaultView().jQElement().hide();
        this._leftItemsDiv.prepend(aBarButtonItem.defaultView().jQElement());
        aBarButtonItem.defaultView().jQElement().show();
    },

    removeRightItems: function() {
        this._rightItemsDiv.children().detach();
    },

    addRightItems: function(rightButtonItems, animated) {
        if (rightButtonItems !== null && rightButtonItems.length > 0)
        {
            var barButtonItem1 = null;
            for (var i = 0; i < rightButtonItems.length; i++)
            {
                barButtonItem1 = rightButtonItems[i];
                this.addRightItem(barButtonItem1);
            }

            if (animated)
            {
                var thisRightItemsDiv = this._rightItemsDiv;
                this._rightItemsDiv.fadeIn(200);
            }
            else
            {
                this._rightItemsDiv.show();
            }
        }
    },

    addRightItem: function(aBarButtonItem) {
        aBarButtonItem.defaultView().jQElement().show();
        this._rightItemsDiv.prepend(aBarButtonItem.defaultView().jQElement());
    },

    addLeftItems: function(leftButtonItems, animated) {
        var topItem = this._stack.head();

        if (leftButtonItems !== null && leftButtonItems.length > 0)
        {
            if (topItem.leftItemsSupplementBackButton() && !topItem.hidesBackButton())
            {
                this.addBackButtonItem();
            }

            var barButtonItem1 = null;
            for (var i = 0; i < leftButtonItems.length; i++)
            {
                barButtonItem1 = leftButtonItems[i];
                this.addLeftItem(barButtonItem1);
            }
        }
        else
        {
            if (!topItem.hidesBackButton())
            {
                this.addBackButtonItem();
            }
        }

        if (animated)
        {
            var thisLeftItemsDiv = this._leftItemsDiv;
            this._leftItemsDiv.fadeIn(200);
        }
        else
        {
            this._leftItemsDiv.show();
        }
    },

    addLeftItem: function(aBarButtonItem) {
        aBarButtonItem.defaultView().jQElement().show();
        this._leftItemsDiv.append(aBarButtonItem.defaultView().jQElement());
    },

    addBackButtonItem: function() {
        var topItem = this._stack.head();
        if (this._backBarButtonItem !== null && !topItem.hidesBackButton())
        {
            this._leftItemsDiv.prepend(this._backBarButtonItem.jQElement());
        }
    },

    updateButtonsTheme: function(dataTheme) {
        this.jQElement().find("a").buttonMarkup({"theme" : dataTheme});
    },

    parseBarStyleToDataTheme: function(barStyle) {
        if (barStyle === APT.BarStyle.Default)
        {
            return "b";
        }

        return "a";
    },

});


