/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.PopoverController = Class.$define("APT.PopoverController", {
    __init__: function() {
        this._viewController = null;
        this._delegate = null;
        this._view = null;
        this._visible = false;
        this._view = new APT.View();
        this._view.setBackgroundColor(APT.Color.Transparent);
        var popoverController = this;
        var eventHandler = function() {
            popoverController._visible = false;
            popoverController._view.jQElement().hide();
        };

        this._view.jQElement().on("click", eventHandler);

        this._view.jQElement().attr("data-role", "dialog");
    },

    $copy: function() {
        var copyObject = new APT.PopoverController();
        copyObject._viewController = this._viewController;
        copyObject._delegate = this._delegate;
        copyObject._view = this._view;
        copyObject._visible = this._visible;
        return copyObject;
    },

    initWithContentViewController: function(aViewController) {
        var popoverController = new APT.PopoverController();
        popoverController.setContentViewController(aViewController);
        return popoverController;
    },

    setContentViewController: function(aViewController) {
        this.setContentViewControllerAnimated(aViewController, false);
    },

    setContentViewControllerAnimated: function(aViewController, animated) {
        if (this._viewController !== aViewController)
        {
            if (this._viewController !== null)
            {
                if (this.popoverVisible())
                {
                    var animation = APT.Animation.beginAnimationsContext(null, null);
                    animation.animateAlpha(this._view, 0.5);
                    animation.setAnimationDuration(400);
                    animation.setAnimationDelegate(this);
                    animation.setAnimationDidStopSelector("executeAnimation");
                    animation.commitAnimations();
                }
                else
                {
                    var point = APT.Point.updateFromCSS((this._view.subviews()[0]).jQElement());
                    var size = this.popoverContentSize();
                    (this._view.subviews()[0]).removeFromSuperview();
                    this._view.addSubview(aViewController.view());
                    APT.Point.updateCSS((this._view.subviews()[0]).jQElement(), point);
                    this.setPopoverContentSize(size);
                }
            }
            else
            {
                this._view.addSubview(aViewController.view());
            }

            this._viewController = aViewController;
            this._view.setViewController(aViewController);
        }
    },

    executeAnimation: function() {
        var point = APT.Point.updateFromCSS((this._view.subviews()[0]).jQElement());
        var size = this.popoverContentSize();
        (this._view.subviews()[0]).removeFromSuperview();
        this._view.addSubview(this._viewController.view());
        APT.Point.updateCSS((this._view.subviews()[0]).jQElement(), point);
        this.setPopoverContentSize(size);
        var animation = APT.Animation.beginAnimationsContext(null, null);
        animation.animateAlpha(this._view, 1);
        animation.setAnimationDuration(400);
        animation.commitAnimations();
    },

    contentViewController: function() {
        return this._viewController;
    },

    popoverContentSize: function() {
        return APT.Size.updateFromCSS((this._view.subviews()[0]).jQElement());
    },

    setPopoverContentSize: function(aSize) {
        aSize = aSize.$copy();
        APT.Size.updateCSS((this._view.subviews()[0]).jQElement(), aSize);
    },

    setPopoverContentSizeAnimated: function(aSize, animated) {
        aSize = aSize.$copy();
        if (animated)
        {
            var animation = APT.Animation.beginAnimationsContext(null, null);
            animation.setAnimatableProperty(this._view.subviews()[0], "width", aSize.width);
            animation.setAnimatableProperty(this._view.subviews()[0], "height", aSize.height);
            animation.setAnimationDuration(1000);
            animation.commitAnimations();
        }
        else
        {
            APT.Size.updateCSS(this._view.jQElement(), aSize);
        }
    },

    delegate: function() {
        return this._delegate;
    },

    setDelegate: function(aDelegate) {
        this._delegate = aDelegate;
    },

    dismissPopoverAnimated: function(animated) {
        this._visible = false;
        this._view.jQElement().hide();
    },

    popoverVisible: function() {
        return this._visible;
    },

    presentPopoverFromRectInViewPermittedArrowDirectionsAnimated: function(aRect, aView, direction, animated) {
        aRect = aRect.$copy();
        this._visible = true;
        var point = aRect.origin;
        point.y = point.y + aRect.size.height + 20;
        APT.Point.updateCSS((this._view.subviews()[0]).jQElement(), point);
        aView.jQElement().append(this._view.jQElement());
        this._view.jQElement().show();
        this._view.addSubview(this._viewController.view());
    },

});


