/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.View = Class.$define("APT.View");

APT.ScrollView = Class.$define("APT.ScrollView", APT.View, {
    __init__: function() {
        this._delegate = null;
        this._jqContent = null;
        this.$super();
    },

    $copy: function() {
        var copyObject = new APT.ScrollView();
        copyObject._delegate = this._delegate;
        copyObject._jqContent = this._jqContent;
        return copyObject;
    },

    getRenderMarkup: function() {
        return $(APT.Global.toStaticHTML("<div class=\"apt_scrollview\" data-viewtype=\"ScrollView\" data-role=\"scrollview\"><div class=\"apt_scrollview2\"></div></div>"));
    },

    setJQElement: function(jqelement) {
        this._jqElement = jqelement.trigger("create");
        this._jqContent = this._jqElement.children(1);

        this.setProperties();
    },

    setScrollEnabled: function(enabled) {
        if (enabled)
        {
            this.jQElement().css("overflow", "auto");
        }
        else
        {
            this.jQElement().css("overflow", "hidden");
        }
    },

    scrollEnabled: function() {
        var scroll = this.jQElement().css("overflow");
        switch(scroll.valueOf())
        {
            case "scroll":
            case "auto":
                return true;
                break;
            case "hidden":
                return false;
                break;
        }
    },

    setContentOffsetAnimated: function(point, animated) {
        point = point.$copy();
        if (animated)
        {
            var animation = APT.Animation.beginAnimationsContext(null, null);
            animation.setAnimatableProperty(this, "scrollTop", parseFloat(point.y));
            animation.setAnimatableProperty(this, "scrollLeft", parseFloat(point.x));
            animation.setAnimationDuration(1000);

            animation.commitAnimations();
        }
        else
        {
            this.jQElement().scrollTop(parseInt(point.y));
            this.jQElement().scrollLeft(parseInt(point.x));
        }
    },

    setContentOffset: function(point) {
        point = point.$copy();
        this.jQElement().scrollTop(parseInt(point.y));
        this.jQElement().scrollLeft(parseInt(point.x));
    },

    contentOffset: function() {
        var x = this.jQElement().scrollLeft();
        var y = this.jQElement().scrollTop();
        return new APT.Point(parseInt(x), parseInt(y));
    },

    setContentSize: function(size) {
        size = size.$copy();
        APT.Size.updateCSS(this._jqContent, size);
    },

    contentSize: function() {
        return APT.Size.updateFromCSS(this._jqContent);
    },

    setDelegate: function(aDelegate) {
        this._delegate = aDelegate;
    },

    delegate: function() {
        return this._delegate;
    },

    insertSubviewAtIndex: function(aView, anIndex) {
        var len = this.subviews().length;
        if (aView !== null && aView !== this && anIndex >= 0 && anIndex <= len)
        {
            if (aView.superview() !== null)
            {
                aView.removeFromSuperview();
            }

            var notifyFlag = this.isInDom() && !aView.isInDom();

            if (notifyFlag)
            {
                aView.viewWillAppear(false);
            }

            if (anIndex === len)
            {
                this._jqContent.append(aView.jQElement());
                this._subviews.push(aView);
            }
            else
            {
                aView.jQElement().insertBefore((this.subviews()[anIndex]).jQElement());
                this._subviews.splice(anIndex, 0, aView);
            }

            if (notifyFlag)
            {
                aView.viewDidAppear(false);
            }

            aView._superview = this;
            aView.setNeedsLayout();
        }
    },

    insertSubviewAtIndexWithoutNotifying: function(aView, anIndex) {
        var len = this._subviews.length;
        if (aView !== null && aView !== this && anIndex >= 0 && anIndex <= len)
        {
            if (aView.superview() !== null)
            {
                aView.removeFromSuperviewWithoutNotifying();
            }

            if (anIndex === len)
            {
                this._jqContent.append(aView.jQElement());
                this._subviews.push(aView);
            }
            else
            {
                aView.jQElement().insertBefore((this._subviews[anIndex]).jQElement());
                this._subviews.splice(anIndex, 0, aView);
            }

            aView._superview = this;
            aView.setNeedsLayout();
        }
    },

    scrollRectToVisible: function(rect, animated) {
        rect = rect.$copy();
        var point = this.contentOffset();

        var x = 0;
        var y = 0;

        if (point.x < rect.origin.x)
        {
            x = parseInt(rect.origin.x + rect.size.width - this.frame().size.width);
        }
        else
        {
            if (point.x > rect.origin.x)
            {
                x = parseInt(rect.origin.x);
            }
        }

        if (point.y < rect.origin.y)
        {
            y = parseInt(rect.origin.y + rect.size.height - this.frame().size.height);
        }
        else
        {
            if (point.y > rect.origin.y)
            {
                y = parseInt(rect.origin.y);
            }
        }

        this.setContentOffsetAnimated(new APT.Point(x, y), animated);
    },

    showsHorizontalScrollIndicator: function() {
        return true;
    },

    showsVerticalScrollIndicator: function() {
        return true;
    },

    setUserInteractionEnabled: function(enabled) {
        this.setScrollEnabled(enabled);
    },

    isUserInteractionEnabled: function() {
        return this.scrollEnabled();
    },

});


