/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.View = Class.$define("APT.View");
APT.Control = Class.$define("APT.Control", APT.View);

APT.SegmentedControl = Class.$define("APT.SegmentedControl", APT.Control, {
    __init__: function() {
        this._items = new Array();
        this._autoWidth = true;
        this.$super();
    },

    $copy: function() {
        var copyObject = new APT.SegmentedControl();
        copyObject._items = this._items;
        copyObject._autoWidth = this._autoWidth;
        return copyObject;
    },

    getRenderMarkup: function() {
        return $(APT.Global.toStaticHTML("<div class=\"apt_segmentedControl ui-corner-all\"></div>"));
    },

    initWithItems: function(items) {
        var aButton = null;
        for (var i = 0; i < items.length; i++)
        {
            aButton = this.createSegmentItem(items[i]);
            if (aButton !== null)
            {
                this._items.push(aButton);
            }
        }

        if (this._autoWidth)
        {
            this.setFrame(APT.Rect.New(0, 0, 100 * items.length, 44));
        }

        return this;
    },

    setImage_forSegmentAtIndex: function(image, segment) {
        if (this.numberOfSegments() > 0)
        {
            segment = this.standardizeSegmentIndex(segment);
            var segmentItem = this._items[segment];
            segmentItem.setImage(image);
        }
    },

    imageForSegmentAtIndex: function(segment) {
        if (this._items.length > 0)
        {
            segment = this.standardizeSegmentIndex(segment);
            return (this._items[segment]).image();
        }
        else
        {
            return null;
        }
    },

    setTitle_forSegmentAtIndex: function(title, segment) {
        if (this.numberOfSegments() > 0)
        {
            if (title === null)
            {
                title = "";
            }

            segment = this.standardizeSegmentIndex(segment);
            var segmentItem = this._items[segment];
            segmentItem.setTitle(title);
        }
    },

    titleForSegmentAtIndex: function(segment) {
        if (this._items.length > 0)
        {
            segment = this.standardizeSegmentIndex(segment);
            return (this._items[segment]).title();
        }
        else
        {
            return null;
        }
    },

    setSelectedSegmentIndex: function(segment) {
        this.deselectItems();
        if (segment >= 0 && segment < this._items.length)
        {
            (this._items[segment]).jQElement().addClass("ui-btn-active");
        }

        this.sendActionsForControlEvents(APT.ControlEvent.ValueChanged);
    },

    selectedSegmentIndex: function() {
        var result = this.jQElement().children(".ui-btn-active").index();
        if (result === -1)
        {
            result = APT.SegmentedControl.NoSegment;
        }

        return result;
    },

    numberOfSegments: function() {
        return this._items.length;
    },

    setFrame: function(aFrame) {
        aFrame = aFrame.$copy();
        this._autoWidth = false;
        this.$super(aFrame);
    },

    fireValueChangedOnSegmentClick: function(sender) {
        if (this.isEnabled())
        {
            this.deselectItems();
            (sender).jQElement().addClass("ui-btn-active");
            this.sendActionsForControlEvents(APT.ControlEvent.ValueChanged);
        }
    },

    insertSegmentWithImage_atIndex: function(image, segment) {
        var newSegment = this.createSegmentItem(image);
        if (segment <= 0)
        {
            this._items.splice(0, 0, newSegment);
            this.jQElement().prepend(newSegment.jQElement());
        }
        else
        {
            if (segment > this._items.length - 1)
            {
                this._items.push(newSegment);
                this.jQElement().append(newSegment);
            }
            else
            {
                this._items.splice(segment, 0, newSegment);
                var afterButton = this._items[segment];
                newSegment.jQElement().insertBefore(afterButton.jQElement());
            }
        }

        this.setNeedsLayout();
    },

    insertSegmentWithTitle_atIndex: function(title, segment) {
        var newSegment = this.createSegmentItem(title);
        if (segment <= 0)
        {
            this._items.splice(0, 0, newSegment);
            this.jQElement().prepend(newSegment.jQElement());
        }
        else
        {
            if (segment > this._items.length - 1)
            {
                this._items.push(newSegment);
                this.jQElement().append(newSegment);
            }
            else
            {
                this._items.splice(segment, 0, newSegment);
                var afterButton = this._items[segment];
                newSegment.jQElement().insertBefore(afterButton.jQElement());
            }
        }

        this.setNeedsLayout();
    },

    removeAllSegments: function() {
        this._items.splice(0, this._items.length);
        this.jQElement().children().detach();
        this.setNeedsLayout();
    },

    removeSegmentAtIndex: function(segment) {
        if (this._items.length > 0)
        {
            segment = this.standardizeSegmentIndex(segment);
            (this._items[segment]).jQElement().detach();
            this._items.splice(segment, 1);
            this.setNeedsLayout();
        }
    },

    setEnabled_forSegmentAtIndex: function(enabled, segment) {
        segment = this.standardizeSegmentIndex(segment);
        var aSegmentItem = this._items[segment];
        if (enabled === false)
        {
            aSegmentItem.jQElement().removeClass("ui-btn-active");
        }

        aSegmentItem.setEnabled(enabled);
    },

    isEnabledForSegmentAtIndex: function(segment) {
        segment = this.standardizeSegmentIndex(segment);
        return (this._items[segment]).isEnabled();
    },

    layoutSubviews: function() {
        this.calculateSegmentSize();
        this.setCorners();
    },

    deselectItems: function() {
        this.jQElement().children().removeClass("ui-btn-active");
    },

    calculateSegmentSize: function() {
        var width1 = parseInt(this.frame().size.width);
        var childrenWidth = 75;
        if (!this._autoWidth && width1 > 0)
        {
            childrenWidth = width1 / this.jQElement().children().length;
        }

        var height1 = parseInt(this.frame().size.height);
        if (this._autoWidth || height1 < 1)
        {
            height1 = 44;
        }

        for (var i = 0; i < this._items.length; i++)
        {
            (this._items[i]).setFrame(APT.Rect.New(i * childrenWidth, 0, childrenWidth, height1));
        }
    },

    setCorners: function() {
        var segmentItem = null;
        for (var i = 0; i < this._items.length; i++)
        {
            segmentItem = this._items[i];
            if (i === 0)
            {
                segmentItem.jQElement().addClass("ui-corner-left");
                segmentItem.jQElement().removeClass("ui-corner-right");
            }
            else
            {
                if (i === this._items.length - 1)
                {
                    segmentItem.jQElement().removeClass("ui-corner-left");
                    segmentItem.jQElement().addClass("ui-corner-right");
                }
                else
                {
                    segmentItem.jQElement().removeClass("ui-corner-left");
                    segmentItem.jQElement().removeClass("ui-corner-right");
                }
            }
        }
    },

    createSegmentItem: function(segmentItem) {
        var aButton = null;
        if (segmentItem instanceof APT.Image)
        {
            aButton = new APT.SegmentButton();
            aButton.setImage(segmentItem);
        }
        else
        {
            if (segmentItem instanceof String || typeof(segmentItem) === "string")
            {
                aButton = new APT.SegmentButton();
                aButton.setTitle(segmentItem);
            }
            else
            {
                aButton = new APT.SegmentButton();
                aButton.setTitle("invalid argument");
            }
        }

        this.jQElement().append(aButton.jQElement());
        aButton.jQElement().addClass("apt_segmentedControl_item");
        aButton.addTarget_action_forControlEvents(this, "fireValueChangedOnSegmentClick", APT.ControlEvent.vmouseup);
        return aButton;
    },

    standardizeSegmentIndex: function(segment) {
        if (segment < 0)
        {
            segment = 0;
        }
        else
        {
            if (segment > this.numberOfSegments() - 1)
            {
                segment = this.numberOfSegments() - 1;
            }
        }

        return segment;
    },

});

APT.SegmentedControl.NoSegment = -1;


