/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.Set = Class.$define("APT.Set", {
    __init__: function() {
        this._array = new Array();
    },

    $copy: function() {
        var copyObject = new APT.Set();
        copyObject._array = this._array;
        return copyObject;
    },

    length: function() {
        return this._array.length;
    },

    initWithObjectsCount: function(aCArray, count) {
        this._array = (aCArray).slice(0, count);
        return this;
    },

    allObjects: function() {
        var ret = new Array();
        ret = this._array.apt_copy();
        return ret;
    },

    anyObject: function() {
        if (this._array !== null && this._array !== undefined && this._array.length > 0)
        {
            return this._array[Math.round(Math.random() * (this._array.length - 1))];
        }
        else
        {
            return null;
        }
    },

    isEqual: function(obj) {
        if (obj !== null && obj !== undefined && obj instanceof APT.Set)
        {
            if (this === obj)
            {
                return true;
            }
            else
            {
                return this._array.apt_isEqual((obj)._array);
            }
        }

        return false;
    },

    addObject: function(element) {
        if (!this.containsObject(element))
        {
            this._array.push(element);
        }
    },

    removeAllObjects: function() {
        this._array.splice(0, this.length());
    },

    removeObject: function(element) {
        var i = 0;
        for (i = 0; i < this.length(); i++)
        {
            if (APT.Global.isEqual(this._array[i], element))
            {
                break;
            }
        }

        if (i !== this.length())
        {
            this._array.splice(i, 1);
        }
    },

    addObjectsFromArray: function(arrayObjects) {
        var i = 0;
        for (i = 0; i < arrayObjects.length; i++)
        {
            this.addObject(arrayObjects[i]);
        }
    },

    initWithArray: function(arr) {
        for (var i = 0; i < arr.length; i++)
        {
            this.addObject(arr[i]);
        }

        return this;
    },

    initWithSet: function(setNew) {
        var arr = setNew.allObjects();

        for (var i = 0; i < arr.length; i++)
        {
            this._array.push(arr[i]);
        }

        return this;
    },

    memberSet: function(objToFind) {
        var newObj = new Object();

        for (var i = 0; i < this._array.length; i++)
        {
            if (APT.Global.isEqual(this._array[i], objToFind))
            {
                newObj = objToFind;
                return newObj;
            }
        }

        return null;
    },

    description: function() {
        var last = this.length();
        var str1;
        var desc = "{(\n";
        for (var i = 0; i < this.length() - 1; i++)
        {
            str1 = this._array[i];
            desc += "    " + str1 + ",\n";
        }

        str1 = this._array[this.length() - 1];
        desc += "    " + str1 + "\n)}";

        return desc;
    },

    setByAddingObject: function(obj) {
        var ret = APT.Set.setWithSet(this);
        ret.addObject(obj);
        return ret;
    },

    setByAddingObjectsFromSet: function(aSet) {
        var ret = APT.Set.setWithSet(this);
        var arr = aSet.allObjects();

        for (var i = 0; i < arr.length; i++)
        {
            ret.addObject(arr[i]);
        }

        return ret;
    },

    setByAddingObjectsFromArray: function(arr) {
        var ret = APT.Set.setWithSet(this);

        for (var i = 0; i < arr.length; i++)
        {
            ret.addObject(arr[i]);
        }

        return ret;
    },

    getEnumerator: function() {
        var keys = this._array;
        var values = new Array();

        for (var i = 0; i < keys.length; i++)
        {
            values.push(this[keys[i]]);
        }

        return APT.Enumerator.New(values);
    },

    containsObject: function(obj) {
        var i = 0;
        for (i = 0; i < this._array.length; i++)
        {
            if (APT.Global.isEqual(this._array[i], obj))
            {
                return true;
            }
        }

        return false;
    },

    isSubsetOfSet: function(aSet) {
        var arr = aSet.allObjects();
        var j = 0;
        for (var i = 0; i < this._array.length; i++)
        {
            for (j = 0; j < arr.length; j++)
            {
                if (APT.Global.isEqual(arr[j], this._array[i]))
                {
                    break;
                }
            }

            if (j === arr.length)
            {
                return false;
            }
        }

        return true;
    },

    intersectsSet: function(aSet) {
        var arr = aSet.allObjects();
        var j = 0;
        var intersect = true;
        for (var i = 0; i < this._array.length; i++)
        {
            for (j = 0; j < arr.length; j++)
            {
                if (APT.Global.isEqual(arr[j], this._array[i]))
                {
                    return intersect;
                }
            }

            if (j === arr.length)
            {
                return !intersect;
            }
        }

        return !intersect;
    },

    makeObjectsPerformSelector: function(aSelector) {

        this._array.every(function(aValue, aIndex, aSetArray) {
            aValue[aSelector]();
            return true;
        });
    },

    makeObjectsPerformSelector_withObject: function(aSelector, anObject) {

        this._array.every(function(aValue, aIndex, aSetArray) {
            aValue[aSelector](anObject);
            return true;
        });
    },

    setValueForKey: function(value, key) {
        if (value === undefined || value === null)
        {
            delete this[key];
        }
        else
        {
            this[key] = value;
        }
    },

    valueForKey: function(key) {
        if (this[key] === undefined)
        {
            return null;
        }

        return this[key];
    },

});

APT.Set.NewWithObjects = function(arr) {
    var ret = new APT.Set();

    for (var i = 0; i < arr.length; i++)
    {
        if (ret._array.indexOf(arr[i]) === -1)
        {
            ret._array.push(arr[i]);
        }
    }

    return ret;
};

APT.Set.setWithObjectsCount = function(aCArray, count) {
    var ret = new APT.Set();
    ret._array = (aCArray).slice(0, count);
    return ret;
};

APT.Set.setWithArray = function(arr) {
    var ret = new APT.Set();
    for (var i = 0; i < arr.length; i++)
    {
        ret.addObject(arr[i]);
    }

    return ret;
};

APT.Set.setWithObject = function(obj) {
    var ret = new APT.Set();
    ret._array.push(obj);
    return ret;
};

APT.Set.setWithSet = function(newSet) {
    var ret = new APT.Set();
    var arr = newSet.allObjects();

    for (var i = 0; i < arr.length; i++)
    {
        ret._array.push(arr[i]);
    }

    return ret;
};


