/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.View = Class.$define("APT.View");

APT.TabBar = Class.$define("APT.TabBar", APT.View, {
    __init__: function() {
        this._items = new Array();
        this._delegate = null;
        this._userInteractionEnabled = true;
        this.$super();
    },

    $copy: function() {
        var copyObject = new APT.TabBar();
        copyObject._items = this._items;
        copyObject._delegate = this._delegate;
        copyObject._userInteractionEnabled = this._userInteractionEnabled;
        return copyObject;
    },

    getRenderMarkup: function() {
        return $(APT.Global.toStaticHTML("<div data-role=\"navbar\"><ul class=\"style_tabbar\"></ul></div>"));
    },

    setJQElement: function(jqElement) {
        this._jqElement = jqElement.navbar();
    },

    setItems: function(items) {
        this._items = items;
        this._jqElement.find("ul").children().detach();

        var percentageWidth = 100;
        if (items.length > 0)
        {
            percentageWidth = parseInt(100 / items.length);
        }

        var aTabBarItem = null;
        var liJQuery = null;

        for (var i = 0; i < this._items.length; i++)
        {
            aTabBarItem = this._items[i];
            this.setHandlerSelectionEventToTabBarItem(aTabBarItem);
            if (i === this._items.length - 1)
            {
                percentageWidth = 100 - percentageWidth * (this._items.length - 1);
            }

            $(aTabBarItem.defaultView().jQElement()[0]).css("width", new Number(percentageWidth).toString() + "%");
            this.addSubview(aTabBarItem.defaultView());
        }

        this.setNeedsLayout();
    },

    items: function() {
        return this._items.slice(0);
    },

    setDelegate: function(aDelegate) {
        this._delegate = aDelegate;
    },

    delegate: function() {
        return this._delegate;
    },

    selectedItem: function() {
        var result = null;

        for (var i = 0; i < this._items.length; i++)
        {
            if ((this._items[i]).isSelected())
            {
                result = this._items[i];
                break;
            }
        }

        return result;
    },

    setSelectedItem: function(selectedItem) {
        this.deselectItems();
        if (selectedItem !== null && selectedItem !== undefined)
        {
            selectedItem.setSelected(true);
        }
    },

    processNode: function(markup, element) {
        var aux = element.children().clone();

        var items = new Array();
        APT.Global.processElement(aux, items);

        this.setJQElement(markup);
        this.setItems(items);
    },

    setUserInteractionEnabled: function(enabled) {
        this._userInteractionEnabled = enabled;
        for (var i = 0; i < this._items.length; i++)
        {
            (this._items[i]).defaultView().jQElement().prop("disabled", !enabled);
        }
    },

    isUserInteractionEnabled: function() {
        return this._userInteractionEnabled;
    },

    deselectItems: function() {
        for (var i = 0; i < this._items.length; i++)
        {
            (this._items[i]).setSelected(false);
        }
    },

    layoutSubviews: function() {
        var height = parseInt(this.frame().size.height);
        if (height === 0)
        {
            height = APT.TabBarController._tabBarDefaultHeight;
        }

        this.jQElement().find("ul").css("height", height);
        this.jQElement().find("li").css("height", height);
        var tabItem = null;
        var tabBarHeight = parseInt(this.frame().size.height);
        for (var i = 0; i < this._items.length; i++)
        {
            tabItem = this._items[i];
            tabItem.defaultView().setHeight(height);
        }
    },

    insertSubviewAtIndex: function(aView, anIndex) {
        var len = this.subviews().length;
        if (aView !== null && aView !== this && anIndex >= 0 && anIndex <= len)
        {
            if (aView.superview() !== null)
            {
                aView.removeFromSuperview();
            }

            var notifyFlag = this.isInDom() && !aView.isInDom();

            if (notifyFlag)
            {
                aView.viewWillAppear(false);
            }

            if (anIndex === len)
            {
                this.jQElement().find("ul").append($(aView.jQElement()[0]));
                this._subviews.push(aView);
            }
            else
            {
                aView.jQElement().insertBefore((this.subviews()[anIndex]).jQElement());
                this._subviews.splice(anIndex, 0, aView);
            }

            if (notifyFlag)
            {
                aView.viewDidAppear(false);
            }

            aView._superview = this;
            aView.setNeedsLayout();
        }
    },

    setHandlerSelectionEventToTabBarItem: function(aTabBarItem) {
        var thisTabBar = this;

        aTabBarItem.defaultView().jQElement().on("vclick", function(event) {
            if (thisTabBar._userInteractionEnabled)
            {
                var item;
                for (var i = 0; i < thisTabBar.items().length; i++)
                {
                    if (thisTabBar.items()[i].defaultView().jQElement()[0] === this)
                    {
                        item = thisTabBar.items()[i];
                        break;
                    }
                }

                thisTabBar.setSelectedItem(item);
                if (thisTabBar.delegate() !== null)
                {
                    thisTabBar.delegate().tabBar_didSelectItem(thisTabBar, item);
                }
            }

            event.stopPropagation();
        });
    },

});


