/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.View = Class.$define("APT.View");
APT.Control = Class.$define("APT.Control", APT.View);

APT.TabBarButtonDefault = Class.$define("APT.TabBarButtonDefault", APT.Control, {
    __init__: function() {
        this._image = null;
        this._title = null;
        this._tabBarSystemItem = -1;
        this._paddingHeightMultiplier = 0.1;
        this.$super();
        this.setImage(null);
        this.setTitle(null);
    },

    $copy: function() {
        var copyObject = new APT.TabBarButtonDefault();
        copyObject._image = this._image;
        copyObject._title = this._title;
        copyObject._tabBarSystemItem = this._tabBarSystemItem;
        copyObject._paddingHeightMultiplier = this._paddingHeightMultiplier;
        return copyObject;
    },

    getRenderMarkup: function() {
        return $(APT.Global.toStaticHTML("<li class=\"style_tabbarbuttondefault\"><a data-role=\"button\" ></a></li>"));
    },

    setJQElement: function(jqElement) {
        this._jqElement = jqElement;
        this.jQElement().find("a").button();
        this.jQElement().find("a").buttonMarkup({"icon" : "star"});
        this.jQElement().find("a").buttonMarkup({"iconpos" : "top"});
        this.jQElement().find("a").buttonMarkup({"iconshadow" : false});
        this.jQElement().find("a").buttonMarkup({"corners" : false});

        this.ajustStylesInnerElements();
    },

    initWithTitle_image_tag: function(title, image, tag) {
        this.setTitle(title);
        this.setImage(image);
        this.setTag(tag);
        return this;
    },

    initWithTabBarSystemItem_tag: function(systemItem, tag) {
        this._tabBarSystemItem = -1;
        var iconContainer = this.jQElement().find("a").find(".ui-icon");
        var textContainer = this.jQElement().find("a").find(".ui-btn-text");
        switch(systemItem)
        {
            case APT.TabBarSystemItem.More:
                iconContainer.css("background-image", "");
                iconContainer.css("background-color", "");
                iconContainer.addClass("apt_tabBarItemIcon_more");
                textContainer.text("More");
                this._tabBarSystemItem = APT.TabBarSystemItem.More;
                break;
            case APT.TabBarSystemItem.Favorites:
                iconContainer.css("background-image", "");
                iconContainer.css("background-color", "");
                iconContainer.addClass("apt_tabBarItemIcon_favorites");
                textContainer.text("Favorites");
                this._tabBarSystemItem = APT.TabBarSystemItem.Favorites;
                break;
            case APT.TabBarSystemItem.Featured:
                iconContainer.css("background-image", "");
                iconContainer.css("background-color", "");
                iconContainer.addClass("apt_tabBarItemIcon_featured");
                textContainer.text("Featured");
                this._tabBarSystemItem = APT.TabBarSystemItem.Featured;
                break;
            case APT.TabBarSystemItem.TopRated:
                iconContainer.css("background-image", "");
                iconContainer.css("background-color", "");
                iconContainer.addClass("apt_tabBarItemIcon_topRated");
                textContainer.text("TopRated");
                this._tabBarSystemItem = APT.TabBarSystemItem.TopRated;
                break;
            case APT.TabBarSystemItem.Recents:
                iconContainer.css("background-image", "");
                iconContainer.css("background-color", "");
                iconContainer.addClass("apt_tabBarItemIcon_recents");
                textContainer.text("Recents");
                this._tabBarSystemItem = APT.TabBarSystemItem.Recents;
                break;
            case APT.TabBarSystemItem.Contacts:
                iconContainer.css("background-image", "");
                iconContainer.css("background-color", "");
                iconContainer.addClass("apt_tabBarItemIcon_contacts");
                textContainer.text("Contacts");
                this._tabBarSystemItem = APT.TabBarSystemItem.Contacts;
                break;
            case APT.TabBarSystemItem.History:
                iconContainer.css("background-image", "");
                iconContainer.css("background-color", "");
                iconContainer.addClass("apt_tabBarItemIcon_history");
                textContainer.text("History");
                this._tabBarSystemItem = APT.TabBarSystemItem.History;
                break;
            case APT.TabBarSystemItem.Bookmarks:
                iconContainer.css("background-image", "");
                iconContainer.css("background-color", "");
                iconContainer.addClass("apt_tabBarItemIcon_bookmarks");
                textContainer.text("Bookmarks");
                this._tabBarSystemItem = APT.TabBarSystemItem.Bookmarks;
                break;
            case APT.TabBarSystemItem.Search:
                iconContainer.css("background-image", "");
                iconContainer.css("background-color", "");
                iconContainer.addClass("apt_tabBarItemIcon_search");
                textContainer.text("Search");
                this._tabBarSystemItem = APT.TabBarSystemItem.More;
                break;
            case APT.TabBarSystemItem.Downloads:
                iconContainer.css("background-image", "");
                iconContainer.css("background-color", "");
                iconContainer.addClass("apt_tabBarItemIcon_downloads");
                textContainer.text("Downloads");
                this._tabBarSystemItem = APT.TabBarSystemItem.Downloads;
                break;
            case APT.TabBarSystemItem.MostRecent:
                iconContainer.css("background-image", "");
                iconContainer.css("background-color", "");
                iconContainer.addClass("apt_tabBarItemIcon_mostRecent");
                textContainer.text("Most Recent");
                this._tabBarSystemItem = APT.TabBarSystemItem.MostRecent;
                break;
            case APT.TabBarSystemItem.MostViewed:
                iconContainer.css("background-image", "");
                iconContainer.css("background-color", "");
                iconContainer.addClass("apt_tabBarItemIcon_mostViewed");
                textContainer.text("Most Viewed");
                this._tabBarSystemItem = APT.TabBarSystemItem.MostViewed;
                break;
            default:
                iconContainer.css("background-image", "none");
                iconContainer.css("background-color", "transparent");
                textContainer.text("");
                break;
        }

        this._image = null;
        this._title = null;
        this.setTag(tag);
        return this;
    },

    setHeight: function(aHeight) {
        var cssHeight = new String(aHeight) + "px";
        this.jQElement().find("a").css("height", cssHeight);

        var buttonInner = this.jQElement().find("a").children(".ui-btn-inner");
        buttonInner.css("height", cssHeight);

        var buttonText = buttonInner.children(".ui-btn-text");
        var textHeight = 18;
        var topMargin = parseInt(cssHeight) - textHeight;
        buttonText.css("top", new Number(topMargin).toString() + "px");

        var buttonIcon = buttonInner.children(".ui-icon");
        var paddingSize = parseInt(topMargin * this._paddingHeightMultiplier);
        buttonIcon.css("height", new Number(topMargin - paddingSize * 2).toString() + "px");
        buttonIcon.css("padding-top", new Number(paddingSize).toString() + "px");
        buttonIcon.css("padding-bottom", new Number(paddingSize).toString() + "px");
    },

    setTitle: function(title) {
        if (!this.isTabBarSystemItem())
        {
            if (title !== null && title !== undefined)
            {
                this._title = title;
                this.jQElement().find("a").find(".ui-btn-text").text(title);
            }
            else
            {
                this._title = null;
                this.jQElement().find("a").find(".ui-btn-text").text("");
            }
        }
    },

    title: function() {
        return this._title;
    },

    setImage: function(image) {
        if (!this.isTabBarSystemItem())
        {
            var iconContainer = this.jQElement().find("a").find(".ui-icon");
            if (image !== null && image !== undefined)
            {
                this._image = image;
                iconContainer.css("background-image", "url(" + image.name() + ")");
                iconContainer.css("background-color", "transparent");
            }
            else
            {
                this._image = null;
                iconContainer.css("background-image", "none");
                iconContainer.css("background-color", "transparent");
            }
        }
    },

    image: function() {
        return this._image;
    },

    ajustStylesInnerElements: function() {
        var anchor = this.jQElement().find("a");
        anchor.css("margin-top", "0px");
        anchor.css("margin-bottom", "0px");
        anchor.css("margin-right", "0px");
        anchor.css("margin-left", "0px");
        anchor.css("pading-top", "0px");
        anchor.css("pading-bottom", "0px");
        anchor.css("pading-right", "0px");
        anchor.css("pading-left", "0px");
        anchor.css("-webkit-box-shadow", "none");
        anchor.css("box-shadow", "none");

        var buttonInner = this.jQElement().find("a").children(".ui-btn-inner");
        buttonInner.css("padding-top", "0px");
        buttonInner.css("padding-bottom", "0px");
        buttonInner.css("padding-left", "0px");
        buttonInner.css("padding-right", "0px");
        buttonInner.css("margin-top", "0px");
        buttonInner.css("margin-bottom", "0px");
        buttonInner.css("margin-left", "0px");
        buttonInner.css("margin-right", "0px");

        var buttonIcon = buttonInner.children(".ui-icon");
        buttonIcon.css("background-position", "0px 0px");
        buttonIcon.css("top", "0px");
        buttonIcon.css("left", "0px");
        buttonIcon.css("text-align", "center");
        var paddingPercentage = parseInt(100 * this._paddingHeightMultiplier);
        buttonIcon.css("padding-left", new Number(paddingPercentage).toString() + "%");
        buttonIcon.css("padding-right", new Number(paddingPercentage).toString() + "%");
        buttonIcon.css("width", new Number(100 - paddingPercentage * 2).toString() + "%");
        buttonIcon.css("margin-top", "0px");
        buttonIcon.css("margin-bottom", "0px");
        buttonIcon.css("margin-left", "0px");
        buttonIcon.css("margin-right", "0px");
        buttonIcon.css("position", "absolute");
        buttonIcon.css("background-position", "center center");
        buttonIcon.css("background-origin", "content-box");
        buttonIcon.css("background-size", "contain");
        buttonIcon.css("-webkit-border-radius", "0px");
        buttonIcon.css("border-radius", "0px");

        var buttonText = buttonInner.children(".ui-btn-text");
        buttonText.css("left", "0px");
        buttonText.css("padding-top", "0px");
        buttonText.css("padding-bottom", "0px");
        buttonText.css("padding-left", "0px");
        buttonText.css("padding-right", "0px");
        buttonText.css("margin-top", "0px");
        buttonText.css("margin-bottom", "0px");
        buttonText.css("margin-left", "0px");
        buttonText.css("margin-right", "0px");
        buttonText.css("position", "absolute");
        buttonText.css("width", "100%");
    },

    isTabBarSystemItem: function() {
        return this._tabBarSystemItem >= 0 && this._tabBarSystemItem <= 11;
    },

    processNode: function(markup, element) {
        var button = element.find("a");

        markup.find("a").attr("data-apt-image", button.attr("data-apt-image"));
        markup.find("a").text(button.text());

        this.setJQElement(markup);
    },

});


