/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.View = Class.$define("APT.View");
APT.ScrollView = Class.$define("APT.ScrollView", APT.View);

APT.TableView = Class.$define("APT.TableView", APT.ScrollView, {
    __init__: function() {
        this._style = 0;
        this._delegate = null;
        this._dataSource = null;
        this._arraySections = null;
        this._arraySectionsTitles = null;
        this._rowHeight = 44;
        this._sectionHeaderHeight = 25;
        this._allowsSelection = true;
        this._tableHeaderView = null;
        this.$super();
    },

    $copy: function() {
        var copyObject = new APT.TableView();
        copyObject._style = this._style;
        copyObject._delegate = this._delegate;
        copyObject._dataSource = this._dataSource;
        copyObject._arraySections = this._arraySections;
        copyObject._arraySectionsTitles = this._arraySectionsTitles;
        copyObject._rowHeight = this._rowHeight;
        copyObject._sectionHeaderHeight = this._sectionHeaderHeight;
        copyObject._allowsSelection = this._allowsSelection;
        copyObject._tableHeaderView = this._tableHeaderView;
        return copyObject;
    },

    getRenderMarkup: function() {
        return $(APT.Global.toStaticHTML("<div><div><ul data-role='listview'></ul></div></div>"));
    },

    setJQElement: function(aObject) {
        this.setJQElementWithStyle(aObject, APT.TableViewStyle.Plain);
    },

    setJQElementWithStyle: function(aObject, aStyle) {
        this._jqElement = aObject.page();
        this._jqElement.css("display", "block");
        this._jqElement.css("min-height", "0px");
        this.setScrollEnabled(true);
        this._jqContent = this._jqElement.children().eq(0);
        if (aStyle !== null)
        {
            this._style = aStyle;
        }

        this._jqContent = this._jqElement.children(1);
        if (aStyle === APT.TableViewStyle.Grouped)
        {
            this._jqContent.attr("data-inset", "true");
        }

        this._jqContent.find("ul").listview();

        this.setProperties();
        if (this.jQElement().css("width") === "0px")
        {
            this.jQElement().css("width", "");
        }

        this.cleanDataTable();
    },

    initWithFrame_style: function(aFrame, aTableViewStyle) {
        aFrame = aFrame.$copy();
        this.setJQElementWithStyle(this.jQElement(), aTableViewStyle);
        APT.View.prototype.setFrame.call(this, aFrame);

        return this;
    },

    delegate: function() {
        return this._delegate;
    },

    setDelegate: function(aDelegate) {
        this._delegate = aDelegate;
    },

    dataSource: function() {
        return this._dataSource;
    },

    setDataSource: function(aDataSource) {
        this._dataSource = aDataSource;
    },

    rowHeight: function() {
        return this._rowHeight;
    },

    setRowHeight: function(aHeight) {
        this._rowHeight = aHeight;
    },

    sectionHeaderHeight: function() {
        return this._sectionHeaderHeight;
    },

    setSectionHeaderHeight: function(aHeight) {
        this._sectionHeaderHeight = aHeight;
    },

    allowsSelection: function() {
        return this._allowsSelection;
    },

    setAllowsSelection: function(value) {
        this._allowsSelection = value;
    },

    tableHeaderView: function() {
        return this._tableHeaderView;
    },

    setTableHeaderView: function(aView) {
        if (this._tableHeaderView !== null)
        {
            this.jQElement().find("ul").css("margin-top", "0px");
            this._tableHeaderView.removeFromSuperview();
        }

        if (aView !== null)
        {
            aView.jQElement().addClass("header-table-view");
            this.insertSubviewAtIndex(aView, 0);
        }

        this._tableHeaderView = aView;
    },

    cleanDataTable: function() {
        var tempArr = new Array();
        this._arraySections = [tempArr];
        this._arraySectionsTitles = [""];
        this.jQElement().find("ul").empty();
    },

    addTableViewSection: function(aSectionName, aSectionHeight) {
        var sectionName = $(APT.Global.toStaticHTML("<div><li data-role='list-divider'></li></div>")).trigger("create").children(1);
        sectionName.text(aSectionName.valueOf());
        sectionName.css("height", aSectionHeight);
        this.jQElement().find("ul").append(sectionName);
        this.jQElement().find("ul").listview("refresh");
    },

    getIndexInDOM: function(aIndexPath) {
        var result = 0;
        if (aIndexPath.section() > 0)
        {
            for (var i = 0; i < aIndexPath.section(); i++)
            {
                result = result + this.numberOfRowsInSection(i);
            }
        }

        if (this._style !== APT.TableViewStyle.Plain)
        {
            result = result + aIndexPath.section() + 1;
        }

        result = result + aIndexPath.row();
        return result;
    },

    addCell_ForIndex: function(aCell, aIndex) {
        if (aIndex.section() === this._arraySections.length)
        {
            this._arraySections[aIndex.section()] = new Array();
        }

        var arrayRows = this._arraySections[aIndex.section()];
        var tempLength = arrayRows.length;
        if (aIndex.row() <= tempLength)
        {
            arrayRows.splice(aIndex.row(), 0, aCell);
            var domIndex = this.getIndexInDOM(aIndex);
            if (aIndex.section() === this._arraySections.length - 1 && aIndex.row() === tempLength)
            {
                this.addSubview(aCell);
            }
            else
            {
                var tempCell = this.jQElement().find("ul").children(1).eq(domIndex);
                aCell.jQElement().insertBefore(tempCell);
            }

            this.jQElement().find("ul").listview("refresh");
        }

        aCell.setTableView(this);
    },

    numberOfSections: function() {
        return this._arraySections.length;
    },

    numberOfRowsInSection: function(aSection) {
        var result = 0;
        if (aSection < this._arraySections.length)
        {
            var tempArray = this._arraySections[aSection];
            result = tempArray.length;
        }

        return result;
    },

    cellForRowAtIndexPath: function(aIndexPath) {
        var result = null;
        if (aIndexPath.section() < this._arraySections.length)
        {
            var rowsArray = this._arraySections[aIndexPath.section()];
            if (aIndexPath.row() < rowsArray.length)
            {
                result = rowsArray[aIndexPath.row()];
            }
        }

        return result;
    },

    reloadData: function() {
        if (this._dataSource !== null)
        {
            this.cleanDataTable();
            var aIndexPath;
            var cantSections = 1;
            if (this._dataSource["numberOfSectionsInTableView"] instanceof Function)
            {
                cantSections = this._dataSource.numberOfSectionsInTableView(this);
            }

            var tempSectionHeight = this.sectionHeaderHeight();
            var tempRowHeight = this.rowHeight();
            for (var i = 0; i < cantSections; i++)
            {
                var cantRows = this._dataSource.tableView_numberOfRowsInSection(this, i);
                if (this._delegate !== null && this._delegate["tableView_heightForHeaderInSection"] instanceof Function)
                {
                    tempSectionHeight = this._delegate.tableView_heightForHeaderInSection(this, i);
                }

                if (this._dataSource["tableView_titleForHeaderInSection"] instanceof Function)
                {
                    var sectionName = this._dataSource.tableView_titleForHeaderInSection(this, i);

                    if (sectionName !== null)
                    {
                        this.addTableViewSection(sectionName.valueOf(), tempSectionHeight);
                    }
                }

                for (var j = 0; j < cantRows; j++)
                {
                    aIndexPath = APT.IndexPath.indexPathForRow_inSection(j, i);
                    if (this._delegate !== null && this._delegate["tableView_heightForRowAtIndexPath"] instanceof Function)
                    {
                        tempRowHeight = this._delegate.tableView_heightForRowAtIndexPath(this, aIndexPath);
                    }

                    var aCell = this._dataSource.tableView_cellForRowAtIndexPath(this, aIndexPath);
                    aCell.jQElement().css("height", tempRowHeight);
                    aCell.contentView().jQElement().css("height", tempRowHeight);
                    aCell.jQElement().find("a").css("padding", 0);
                    if (tempRowHeight < 44)
                    {
                        aCell.jQElement().find(".apt_view").css("padding-top", 0);
                        if (aCell.textLabel() !== null)
                        {
                            aCell.textLabel().jQElement().css("padding-top", 0);
                        }

                        if (aCell.detailTextLabel() !== null)
                        {
                            aCell.detailTextLabel().jQElement().css("padding-top", 0);
                        }

                        if (aCell.imageView() !== null)
                        {
                            aCell.imageView().jQElement().css("max-width", tempRowHeight);
                            aCell.imageView().jQElement().css("max-height", tempRowHeight);
                        }
                    }

                    this.addCell_ForIndex(aCell, aIndexPath);
                }
            }
        }
    },

    insertRowsAtIndexPaths_withRowAnimation: function(aIndexPaths, anAnimation) {
        var tempRowHeight = this.rowHeight();
        for (var i = 0; i < aIndexPaths.length; i++)
        {
            var aIndex = aIndexPaths[i];
            if (aIndex.section() < this._arraySections.length)
            {
                if (this._delegate !== null && this._delegate["tableView_heightForRowAtIndexPath"] instanceof Function)
                {
                    tempRowHeight = this._delegate.tableView_heightForRowAtIndexPath(this, aIndex);
                }

                var aCell = this._dataSource.tableView_cellForRowAtIndexPath(this, aIndex);
                aCell.jQElement().css("height", tempRowHeight);
                aCell.jQElement().find("a").css("padding", 0);
                if (tempRowHeight < 44)
                {
                    aCell.jQElement().find(".apt_view").css("padding-top", 0);
                    if (aCell.textLabel() !== null)
                    {
                        aCell.textLabel().jQElement().css("padding-top", 0);
                    }

                    if (aCell.detailTextLabel() !== null)
                    {
                        aCell.detailTextLabel().jQElement().css("padding-top", 0);
                    }

                    if (aCell.imageView() !== null)
                    {
                        aCell.imageView().jQElement().css("max-width", tempRowHeight);
                        aCell.imageView().jQElement().css("max-height", tempRowHeight);
                    }
                }

                this.addCell_ForIndex(aCell, aIndex);
            }
        }
    },

    deleteRowsAtIndexPaths_withRowAnimation: function(aIndexPaths, anAnimation) {
        for (var i = 0; i < aIndexPaths.length; i++)
        {
            var aIndex = aIndexPaths[i];
            if (aIndex.section() < this._arraySections.length)
            {
                var arrayRows = this._arraySections[aIndex.section()];
                if (aIndex.row() < arrayRows.length)
                {
                    var aCell = arrayRows[aIndex.row()];
                    aCell.setTableView(null);
                    arrayRows.splice(aIndex.row(), 1);
                    var domIndex = this.getIndexInDOM(aIndex);
                    var tempCell = this.jQElement().find("ul").children(1).eq(domIndex);
                    tempCell.remove();
                    this.jQElement().find("ul").listview("refresh");
                }
            }
        }
    },

    indexPathForSelectedRow: function() {
        for (var i = 0; i < this._arraySections.length; i++)
        {
            var arrayRows = this._arraySections[i];
            for (var j = 0; j < arrayRows.length; j++)
            {
                var aCell = arrayRows[j];
                if (aCell.isSelected())
                {
                    return APT.IndexPath.indexPathForRow_inSection(j, i);
                }
            }
        }

        return null;
    },

    selectRowAtIndexPath_animated_select: function(aIndex, animated, selected) {
        if (aIndex.section() < this._arraySections.length)
        {
            var arrayRows = this._arraySections[aIndex.section()];
            if (aIndex.row() < arrayRows.length)
            {
                var aCell = arrayRows[aIndex.row()];
                aCell.setSelected(selected);
            }
        }
    },

    selectChange: function(ev) {
        if (ev !== null && this.allowsSelection())
        {
            var aCell = $(ev.currentTarget);
            for (var i = 0; i < this._arraySections.length; i++)
            {
                var rowArr = this._arraySections[i];
                for (var j = 0; j < rowArr.length; j++)
                {
                    var aCelltemp = rowArr[j];
                    if (aCelltemp !== null && aCelltemp.jQElement()[0] === aCell[0])
                    {
                        aCelltemp.setSelected(true);
                        if (this._delegate !== null && this._delegate["tableView_didSelectRowAtIndexPath"] instanceof Function)
                        {
                            var aIndexPath = APT.IndexPath.indexPathForRow_inSection(j, i);
                            this._delegate.tableView_didSelectRowAtIndexPath(this, aIndexPath);
                        }
                    }
                    else
                    {
                        if (aCelltemp.isSelected())
                        {
                            if (this._delegate !== null && this._delegate["tableView_didDeselectRowAtIndexPath"] instanceof Function)
                            {
                                var aIndexPath = APT.IndexPath.indexPathForRow_inSection(j, i);
                                this._delegate.tableView_didDeselectRowAtIndexPath(this, aIndexPath);
                            }

                            aCelltemp.setSelected(false);
                        }
                    }
                }
            }
        }
    },

    style: function() {
        this._style;
    },

    viewWillAppear: function(animated) {
        this.$super(animated);
        this.reloadData();
    },

    setContentSize: function(size) {
        size = size.$copy();
    },

    contentSize: function() {
        return new APT.Size(0, 0);
    },

    dequeueReusableCellWithIdentifier: function() {
        return null;
    },

    insertSubviewAtIndex: function(aView, anIndex) {
        var len = this.subviews().length;
        if (aView !== null && aView !== this && anIndex >= 0 && anIndex <= len)
        {
            if (aView.superview() !== null)
            {
                aView.removeFromSuperview();
            }

            var notifyFlag = this.isInDom() && !aView.isInDom();

            if (notifyFlag)
            {
                aView.viewWillAppear(false);
            }

            if (anIndex === 0)
            {
                if (aView.jQElement()[0] instanceof HTMLLIElement)
                {
                    if (this.jQElement().find("ul").find("li").length === 0)
                    {
                        this.jQElement().find("ul").prepend(aView.jQElement());
                    }
                    else
                    {
                        this.jQElement().find("ul").append(aView.jQElement());
                    }
                }
                else
                {
                    this.jQElement().prepend(aView.jQElement());
                }

                this._subviews.splice(anIndex, 0, aView);
            }
            else
            {
                if (anIndex === len)
                {
                    if (aView.jQElement()[0] instanceof HTMLLIElement)
                    {
                        this.jQElement().find("ul").append(aView.jQElement());
                    }
                    else
                    {
                        this.jQElement().append(aView.jQElement());
                    }

                    this._subviews.push(aView);
                }
                else
                {
                    aView.jQElement().insertBefore((this.subviews()[anIndex]).jQElement());
                    this._subviews.splice(anIndex, 0, aView);
                }
            }

            if (notifyFlag)
            {
                aView.viewDidAppear(false);
            }

            aView._superview = this;
        }
    },

});


