/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.View = Class.$define("APT.View");

APT.TableViewCell = Class.$define("APT.TableViewCell", APT.View, {
    __init__: function() {
        this._contentView = null;
        this._style = 0;
        this._imageView = null;
        this._textLabel = null;
        this._detailTextLabel = null;
        this._table = null;
        this._selected = false;
        this.$super();
    },

    $copy: function() {
        var copyObject = new APT.TableViewCell();
        copyObject._contentView = this._contentView;
        copyObject._style = this._style;
        copyObject._imageView = this._imageView;
        copyObject._textLabel = this._textLabel;
        copyObject._detailTextLabel = this._detailTextLabel;
        copyObject._table = this._table;
        copyObject._selected = this._selected;
        return copyObject;
    },

    getRenderMarkup: function() {
        return $(APT.Global.toStaticHTML("<li data-icon='false'><a></a></li>"));
    },

    setJQElement: function(aObject) {
        this._jqElement = aObject.trigger("create");
        this.setProperties();
        this.jQElement().css("position", "relative");
    },

    init: function() {
        return this.initWithStyle(APT.TableViewCellStyle.Default);
    },

    initWithStyle: function(aTableViewCellStyle) {
        this._contentView = new APT.View();
        this._contentView.setBackgroundColor(APT.Color.Transparent);
        this.addSubview(this._contentView);

        this._textLabel = new APT.Label();
        this._textLabel.jQElement().css("padding-left", 10);

        switch(aTableViewCellStyle)
        {
            case APT.TableViewCellStyle.Default:
                this._imageView = new APT.ImageView();
                this._imageView.jQElement().css("max-width", "44px");
                this._imageView.jQElement().css("max-height", "44px");
                this._imageView.jQElement().css("padding-left", 5);
                this._imageView.jQElement().css("position", "absolute");

                this._textLabel.jQElement().css("padding-top", 12);
                this._textLabel.setFont(APT.Font.boldSystemFontOfSize(12));

                break;
            case APT.TableViewCellStyle.Value1:
                this._textLabel.jQElement().css("position", "absolute");
                this._textLabel.jQElement().css("padding-top", 12);
                this._textLabel.setTextAlignment(APT.TextAlignment.Left);
                this._textLabel.setFont(APT.Font.boldSystemFontOfSize(12));

                this._detailTextLabel = new APT.Label();
                this._detailTextLabel.setTextAlignment(APT.TextAlignment.Right);
                this._detailTextLabel.jQElement().css("padding-top", 12);
                this._detailTextLabel.setTextColor(APT.Color.Blue);
                this._detailTextLabel.font().setSize(9);
                break;
            case APT.TableViewCellStyle.Value2:
                this._textLabel.jQElement().css("position", "absolute");
                this._textLabel.jQElement().css("padding-top", 12);
                this._textLabel.setTextColor(APT.Color.Blue);
                this._textLabel.font().setSize(9);

                this._detailTextLabel = new APT.Label();
                this._detailTextLabel.jQElement().css("position", "absolute");
                this._detailTextLabel.jQElement().css("padding-top", 12);
                this._detailTextLabel.jQElement().css("padding-left", 10);
                this._detailTextLabel.setFont(APT.Font.boldSystemFontOfSize(12));
                break;
            case APT.TableViewCellStyle.Subtitle:
                this._imageView = new APT.ImageView();
                this._imageView.jQElement().css("max-width", "44px");
                this._imageView.jQElement().css("max-height", "44px");
                this._imageView.jQElement().css("padding-left", 5);
                this._imageView.jQElement().css("position", "absolute");

                this._textLabel.jQElement().css("padding-top", 5);
                this._textLabel.setFont(APT.Font.boldSystemFontOfSize(12));

                this._detailTextLabel = new APT.Label();
                this._detailTextLabel.jQElement().css("padding-left", 10);
                this._detailTextLabel.jQElement().css("padding-top", 5);
                this._detailTextLabel.font().setSize(9);
                break;
        }

        this._style = aTableViewCellStyle;
        return this;
    },

    textLabel: function() {
        if (this._contentView !== null && this._contentView.indexOfView(this._textLabel) === -1)
        {
            this._contentView.addSubview(this._textLabel);
            if (this._contentView.indexOfView(this._detailTextLabel) > -1 && (this._style === APT.TableViewCellStyle.Value1 || this._style === APT.TableViewCellStyle.Value2))
            {
                this._contentView.exchangeSubviewAtIndexWithSubviewAtIndex(this._contentView.indexOfView(this._detailTextLabel), this._contentView.indexOfView(this._textLabel));
                if (this._style === APT.TableViewCellStyle.Value2)
                {
                    this._detailTextLabel.jQElement().css("margin-left", this._textLabel.text().length + 60);
                }
            }
        }

        return this._textLabel;
    },

    detailTextLabel: function() {
        if (this._detailTextLabel)
        {
            if (this._contentView !== null && this._contentView.indexOfView(this._detailTextLabel) === -1)
            {
                this._contentView.addSubview(this._detailTextLabel);
                if (this._style === APT.TableViewCellStyle.Value2)
                {
                    this._detailTextLabel.jQElement().css("margin-left", this._textLabel.text().length + 60);
                }
            }
        }

        return this._detailTextLabel;
    },

    imageView: function() {
        if (this._contentView !== null && this._contentView.indexOfView(this._imageView) === -1)
        {
            this._contentView.addSubview(this._imageView);
            if (this._contentView.indexOfView(this._textLabel) > -1)
            {
                if (this._style === APT.TableViewCellStyle.Default)
                {
                    this._textLabel.jQElement().css("margin-left", 54);
                }

                this._contentView.exchangeSubviewAtIndexWithSubviewAtIndex(this._contentView.indexOfView(this._textLabel), this._contentView.indexOfView(this._imageView));
            }

            if (this._style === APT.TableViewCellStyle.Subtitle)
            {
                this._textLabel.jQElement().css("margin-left", 50);
                this._detailTextLabel.jQElement().css("margin-left", 50);
            }
        }

        return this._imageView;
    },

    contentView: function() {
        return this._contentView;
    },

    setContentView: function(content) {
        this._contentView = content;
    },

    setText: function(aText) {
        this.textLabel().setText(aText.valueOf());
    },

    text: function() {
        if (this._textLabel !== null)
        {
            return this._textLabel.text();
        }

        return null;
    },

    accessoryType: function() {
        var result = 0;
        var tempStr = this.jQElement().attr("data-icon");
        switch(tempStr.valueOf())
        {
            case "false":
                result = APT.TableViewCellAccessoryType.None;
                break;
            case "arrow-r":
                result = APT.TableViewCellAccessoryType.DisclosureIndicator;
                break;
            case "check":
                result = APT.TableViewCellAccessoryType.Checkmark;
                break;
        }

        return result;
    },

    setAccessoryType: function(aType) {
        switch(aType)
        {
            case APT.TableViewCellAccessoryType.None:
                this.jQElement().attr("data-icon", "false");
                break;

            case APT.TableViewCellAccessoryType.DisclosureIndicator:
                this.jQElement().attr("data-icon", "arrow-r");
                break;

            case APT.TableViewCellAccessoryType.Checkmark:
                this.jQElement().attr("data-icon", "check");
                break;
        }
    },

    setTableView: function(aTable) {
        this._table = aTable;
        if (aTable !== null)
        {
            var obj = this._table;

            this._jqElement.on("vmousedown", function(event) {
                obj["selectChange"](event);
                event.stopPropagation();
            });
        }
        else
        {
            var obj = this._table;
            this._jqElement.off("vmousedown");
        }
    },

    setSelected: function(aValue) {
        this._selected = aValue;
        if (aValue)
        {
            this.jQElement().addClass("selectedCell");
        }
        else
        {
            this.jQElement().removeClass("selectedCell");
        }
    },

    isSelected: function() {
        return this._selected;
    },

    layoutSubviews: function() {
        this.jQElement().css("width", "");
        this._contentView.jQElement().css("width", this.jQElement().css("width"));
        this._contentView.jQElement().css("height", "");
        if (this._imageView !== null)
        {
            this._imageView.jQElement().css("height", "");
            this._imageView.jQElement().css("width", "");
        }
    },

    viewDidAppear: function(animated) {
        this.setNeedsLayout();
        this.$super(animated);
    },

});


