/*
 * Copyright(C) 2013 Intel Corporation.  All Rights Reserved.
 */



APT.createNamespace("APT");

APT.View = Class.$define("APT.View", {
    __init__: function() {
        this._jqElement = null;
        this._superview = null;
        this._subviews = new Array();
        this._backgroundColor = APT.Color.Black;
        this._tag = 0;
        this._viewController = null;
        this._canvas = null;
        this._bounds = APT.Rect.New(0, 0, 0, 0);
        this._window = null;
        if (this.jQElement() === null)
        {
            this.setJQElement(this.getRenderMarkup());
        }
    },

    $copy: function() {
        var copyObject = new APT.View();
        copyObject._jqElement = this._jqElement;
        copyObject._superview = this._superview;
        copyObject._subviews = this._subviews;
        copyObject._backgroundColor = this._backgroundColor;
        copyObject._tag = this._tag;
        copyObject._viewController = this._viewController;
        copyObject._canvas = this._canvas;
        copyObject._bounds = this._bounds.$copy();
        copyObject._window = this._window;
        return copyObject;
    },

    setCanvas: function() {
        if (this._canvas !== null && !APT.Size.EqualToSize(this.frame().size, this._canvas.frame().size))
        {
            this._canvas.setFrame(this.frame());
        }
    },

    getRenderMarkup: function() {
        return $(APT.Global.toStaticHTML("<div class=\"apt_view\"></div>"));
    },

    isUserInteractionEnabled: function() {
        return !this.jQElement().prop("disabled");
    },

    canvas: function() {
        return this._canvas;
    },

    setUserInteractionEnabled: function(enabled) {
        this.jQElement().prop("disabled", !enabled);
    },

    initWithFrame: function(rect) {
        rect = rect.$copy();
        if (this._jqElement !== null)
        {
            this.setFrame(rect);
            return this;
        }

        return null;
    },

    setWindow: function(newWin) {
        this._window = newWin;
    },

    addSubview: function(child) {
        this.insertSubviewAtIndex(child, this._subviews.length);
    },

    bringSubviewToFront: function(aView) {
        if (aView !== null && aView !== this)
        {
            var index = this.indexOfView(aView);

            if (index > -1 && aView !== this._subviews[this._subviews.length - 1])
            {
                aView.jQElement().insertAfter((this._subviews[this._subviews.length - 1]).jQElement());
                this._subviews.splice(index, 1);
                this._subviews.splice(this._subviews.length, 0, aView);

                aView._superview = this;
                aView.setNeedsLayout();
                aView.setNeedsDisplay();
            }
        }
    },

    sendSubviewToBack: function(aView) {
        if (aView !== null && aView !== this)
        {
            var index = this.indexOfView(aView);

            if (index > -1 && aView !== this._subviews[0])
            {
                aView.jQElement().insertBefore((this._subviews[0]).jQElement());
                this._subviews.splice(index, 1);
                this._subviews.splice(0, 0, aView);

                aView._superview = this;
                aView.setNeedsLayout();
                aView.setNeedsDisplay();
            }
        }
    },

    insertSubviewAtIndex: function(aView, anIndex) {
        var len = this._subviews.length;
        if (aView !== null && aView !== this && anIndex >= 0 && anIndex <= len)
        {
            aView.willMoveToWindow(this._window);
            aView.setWindow(this._window);
            aView.willMoveToSuperview(this);

            if (aView.superview() !== null)
            {
                aView.removeFromSuperview();
            }

            var notifyFlag = this.isInDom() && !aView.isInDom();

            if (notifyFlag)
            {
                aView.viewWillAppear(false);
            }

            if (anIndex === len)
            {
                this._jqElement.append(aView.jQElement());
                this._subviews.push(aView);
            }
            else
            {
                aView.jQElement().insertBefore((this._subviews[anIndex]).jQElement());
                this._subviews.splice(anIndex, 0, aView);
            }

            if (notifyFlag)
            {
                aView.viewDidAppear(false);
            }

            aView._superview = this;

            aView.didMoveToWindow();
            aView.didMoveToSuperview();
            this.didAddSubview(aView);

            aView.setNeedsLayout();
            aView.setNeedsDisplay();
        }
    },

    findViewById: function(viewId) {
        if (this._jqElement === undefined || this._jqElement === null)
        {
            return null;
        }

        if (this._jqElement.attr("id") === viewId)
        {
            if (this instanceof APT.BarButtonDefault)
            {
                return (this)._barButtonItem;
            }
            else
            {
                return this;
            }
        }

        var len = this._subviews.length;
        for (var i = 0; i < len; i++)
        {
            var temp = (this._subviews[i]).findViewById(viewId);
            if (temp !== null)
            {
                return temp;
            }
        }

        return null;
    },

    insertSubviewAboveSubview: function(aView, aSiblingSubview) {
        var index = this.indexOfView(aSiblingSubview);
        if (index >= 0)
        {
            this.insertSubviewAtIndex(aView, index + 1);
        }
    },

    insertSubviewBelowSubview: function(aView, aSiblingSubview) {
        var index = this.indexOfView(aSiblingSubview);
        if (index >= 0)
        {
            this.insertSubviewAtIndex(aView, index);
        }
    },

    removeFromSuperview: function() {
        if (this.superview() !== null && this.superview().jQElement() !== null)
        {
            this.superview().willRemoveSubview(this);

            var index = this._superview.indexOfView(this);
            if (index > -1)
            {
                this._superview._subviews.splice(index, 1);
            }

            this.viewWillDisappear(false);
            this.jQElement().detach();
            this._superview = null;

            this.viewDidDisappear(false);
        }
    },

    isDescendantOfView: function(aView) {
        if (aView !== null)
        {
            var index = aView.indexOfView(this);
            if (index > -1 || this === aView)
            {
                return true;
            }
        }

        return false;
    },

    exchangeSubviewAtIndexWithSubviewAtIndex: function(index1, index2) {
        if (index1 > -1 && index1 < this._subviews.length && index2 > -1 && index2 < this._subviews.length)
        {
            var aView = this._subviews[index1];
            var aView2 = this._subviews[index2];

            aView.removeFromSuperview();
            aView2.removeFromSuperview();

            this.insertSubviewAtIndex(aView2, index1);
            this.insertSubviewAtIndex(aView, index2);
        }
    },

    setFrame: function(aFrame) {
        aFrame = aFrame.$copy();
        this._bounds.size = aFrame.size.$copy();

        this._jqElement.css("position", "absolute");
        APT.Point.updateCSS(this._jqElement, aFrame.origin);
        APT.Size.updateCSS(this._jqElement, aFrame.size);

        if (this._viewController !== null)
        {
            this._viewController.viewWillLayoutSubviews();
        }

        this.layoutSubviews();

        if (this._viewController !== null)
        {
            this._viewController.viewDidLayoutSubviews();
        }

        this.setNeedsDisplay();
    },

    frame: function() {
        var point = APT.Point.updateFromCSS(this._jqElement);
        var size = APT.Size.updateFromCSS(this._jqElement);
        return new APT.Rect(point, size);
    },

    setBounds: function(aRect) {
        aRect = aRect.$copy();
        this._bounds = aRect.$copy();

        var center = this.center();

        var x = center.x - aRect.size.width / 2;
        var y = center.y - aRect.size.height / 2;

        var point = new APT.Point(x, y);
        var size = new APT.Size(aRect.size.width, aRect.size.height);

        this.setFrame(new APT.Rect(point, size));

        this._jqElement.css("position", "absolute");
        APT.Point.updateCSS(this._jqElement, new APT.Point(x, y));
        APT.Size.updateCSS(this._jqElement, aRect.size);

        if (this._viewController !== null)
        {
            this._viewController.viewWillLayoutSubviews();
        }

        this.layoutSubviews();

        if (this._viewController !== null)
        {
            this._viewController.viewDidLayoutSubviews();
        }

        this.setNeedsDisplay();
    },

    bounds: function() {
        return this._bounds;
    },

    window: function() {
        var len = this._subviews.length;
        for (var i = 0; i < len; i++)
        {
            var temp = this._subviews[i];
            var attr = temp._jqElement.attr("data-viewtype");
            if (attr !== undefined && attr === "Window")
            {
                return temp;
            }
        }

        if (this._superview !== null && this._superview !== undefined)
        {
            var attr = this._superview._jqElement.attr("data-viewtype");
            if (attr !== undefined && attr === "Window")
            {
                return this._superview;
            }
            else
            {
                return this._superview.window();
            }
        }

        return null;
    },

    setCenter: function(point) {
        point = point.$copy();
        var frameAux = this.frame();

        var x = point.x - frameAux.size.width / 2;
        var y = point.y - frameAux.size.height / 2;

        this.setFrame(APT.Rect.New(x, y, frameAux.size.width, frameAux.size.height));

        this._jqElement.css("position", "absolute");
        APT.Point.updateCSS(this._jqElement, new APT.Point(x, y));
        APT.Size.updateCSS(this._jqElement, frameAux.size);

        if (this._viewController !== null)
        {
            this._viewController.viewWillLayoutSubviews();
        }

        this.layoutSubviews();

        if (this._viewController !== null)
        {
            this._viewController.viewDidLayoutSubviews();
        }

        this.setNeedsDisplay();
    },

    center: function() {
        var frameAux = this.frame();

        var x = frameAux.origin.x + frameAux.size.width / 2;
        var y = frameAux.origin.y + frameAux.size.height / 2;

        return new APT.Point(x, y);
    },

    viewWithTag: function(aTag) {
        if (this._tag === aTag)
        {
            return this;
        }
        var len = this._subviews.length;
        for (var i = 0; i < len; i++)
        {
            var view = (this._subviews[i]).viewWithTag(aTag);
            if (view !== null)
            {
                return view;
            }
        }

        return null;
    },

    layoutSubviews: function() {
    },

    setAlpha: function(value) {
        this._jqElement.css("opacity", value);
    },

    alpha: function() {
        var alpha = 1;
        if (this._jqElement.css("opacity") !== null)
        {
            alpha = parseFloat(this._jqElement.css("opacity"));
        }

        return alpha;
    },

    setNeedsLayout: function() {
        if (this._viewController !== null)
        {
            this._viewController.viewWillLayoutSubviews();
        }

        this.layoutSubviews();

        if (this._viewController !== null)
        {
            this._viewController.viewDidLayoutSubviews();
        }
    },

    subviews: function() {
        return this._subviews.slice(0);
    },

    setNeedsDisplay: function() {
        if (this.isInDom())
        {
            this.callDrawRect();
        }
    },

    superview: function() {
        return this._superview;
    },

    setTag: function(aTag) {
        this._tag = aTag;
    },

    tag: function() {
        return this._tag;
    },

    setHidden: function(hidden) {
        if (hidden)
        {
            this._jqElement.hide();
        }
        else
        {
            this._jqElement.show();
            this._jqElement.css("display", "block");
        }
    },

    isHidden: function() {
        return !(this._jqElement.is(":visible") || this._jqElement.css("display") === "block") || this._jqElement.css("display") === "none" || this._jqElement.css("visibility") === "hidden";
    },

    setBackgroundColor: function(aColor) {
        this._backgroundColor = aColor;
        if (aColor !== null)
        {
            this.jQElement().css("background-color", aColor.getHtmlValue().valueOf());
        }
    },

    backgroundColor: function() {
        if (this._backgroundColor !== null)
        {
            this._backgroundColor.updateFromCSS(this.jQElement().css("background-color"));
            if (!this._backgroundColor.isEqual(APT.Color.Transparent))
            {
                return this._backgroundColor;
            }
        }

        return null;
    },

    jQElement: function() {
        return this._jqElement;
    },

    setJQElement: function(jqElement) {
        var aux = $("<div></div>");
        aux.append(jqElement);
        this._jqElement = aux.trigger("create").children(1);
        this._jqElement.css("display", "block");
        this.setProperties();
        this.configureTouchEvents();
    },

    configureTouchEvents: function() {
        var _view = this;

        var downevent;
        var upevent;
        var moveevent;
        var cancelevent;

        if (navigator.msPointerEnabled)
        {
            downevent = "MSPointerDown";
            upevent = "MSPointerUp";
            moveevent = "MSPointerMove";
            cancelevent = "MSPointerCancel";
        }
        else
        {
            downevent = "touchstart";
            upevent = "touchend";
            moveevent = "touchmove";
            cancelevent = "touchcancel";
        }

        this.jQElement().on(downevent, function(event) {
            var arrayAux = new Array();
            var touch = null;
            var x = 0;
            var y = 0;

            if (jQuery(event.currentTarget).parents(".apt_scrollview").length > 0)
            {
                x = jQuery(event.currentTarget).parents(".apt_scrollview").scrollLeft();
                y = jQuery(event.currentTarget).parents(".apt_scrollview").scrollTop();
            }
            else
            {
                if (jQuery(event.currentTarget).find(".apt_scrollview").length > 0)
                {
                    x = jQuery(event.currentTarget).find(".apt_scrollview").scrollLeft();
                    y = jQuery(event.currentTarget).find(".apt_scrollview").scrollTop();
                }
            }

            if (event.type === "MSPointerDown")
            {
                touch = new APT.Touch(x + event.originalEvent.pageX, y + event.originalEvent.pageY, _view);
                arrayAux[0] = touch;
            }
            else
            {
                for (var i = 0; i < event.originalEvent.targetTouches.length; i++)
                {
                    touch = new APT.Touch(x + event.originalEvent.targetTouches[i].clientX, y + event.originalEvent.targetTouches[i].clientY, _view);
                    arrayAux[i] = touch;
                }
            }

            var touches = APT.Set.NewWithObjects(arrayAux);
            var e = new APT.Event();
            e._evnt = event;
            e.setTouches(touches);
            _view.touchesBegan_withEvent(touches, e);
            event.stopPropagation();
        });

        this.jQElement().on(moveevent, function(event) {
            var arrayAux = new Array();
            var touch;
            var x = 0;
            var y = 0;

            if (jQuery(event.currentTarget).parents(".apt_scrollview").length > 0)
            {
                x = jQuery(event.currentTarget).parents(".apt_scrollview").scrollLeft();
                y = jQuery(event.currentTarget).parents(".apt_scrollview").scrollTop();
            }
            else
            {
                if (jQuery(event.currentTarget).find(".apt_scrollview").length > 0)
                {
                    x = jQuery(event.currentTarget).find(".apt_scrollview").scrollLeft();
                    y = jQuery(event.currentTarget).find(".apt_scrollview").scrollTop();
                }
            }

            if (event.type === "MSPointerMove")
            {
                touch = new APT.Touch(x + event.originalEvent.pageX, y + event.originalEvent.pageY, _view);
                arrayAux[0] = touch;
            }
            else
            {
                for (var i = 0; i < event.originalEvent.changedTouches.length; i++)
                {
                    touch = new APT.Touch(x + event.originalEvent.changedTouches[i].clientX, y + event.originalEvent.changedTouches[i].clientY, _view);
                    arrayAux[i] = touch;
                }
            }

            var touches = APT.Set.NewWithObjects(arrayAux);
            var e = new APT.Event();
            e._evnt = event;
            e.setTouches(touches);
            _view.touchesMoved_withEvent(touches, e);
            event.stopPropagation();
        });

        this.jQElement().on(upevent, function(event) {
            var arrayAux = new Array();
            var touch;
            var x = 0;
            var y = 0;

            if (jQuery(event.currentTarget).parents(".apt_scrollview").length > 0)
            {
                x = jQuery(event.currentTarget).parents(".apt_scrollview").scrollLeft();
                y = jQuery(event.currentTarget).parents(".apt_scrollview").scrollTop();
            }
            else
            {
                if (jQuery(event.currentTarget).find(".apt_scrollview").length > 0)
                {
                    x = jQuery(event.currentTarget).find(".apt_scrollview").scrollLeft();
                    y = jQuery(event.currentTarget).find(".apt_scrollview").scrollTop();
                }
            }

            if (event.type === "MSPointerUp")
            {
                touch = new APT.Touch(x + event.originalEvent.pageX, y + event.originalEvent.pageY, _view);
                arrayAux[0] = touch;
            }
            else
            {
                for (var i = 0; i < event.originalEvent.changedTouches.length; i++)
                {
                    touch = new APT.Touch(x + event.originalEvent.changedTouches[i].clientX, y + event.originalEvent.changedTouches[i].clientY, _view);
                    arrayAux[i] = touch;
                }
            }

            var touches = APT.Set.NewWithObjects(arrayAux);
            var e = new APT.Event();
            e._evnt = event;
            e.setTouches(touches);
            _view.touchesEnded_withEvent(touches, e);
            event.stopPropagation();
        });

        this.jQElement().on(cancelevent, function(event) {
            var arrayAux = new Array();
            var touch;
            var x = 0;
            var y = 0;

            if (jQuery(event.currentTarget).parents(".apt_scrollview").length > 0)
            {
                x = jQuery(event.currentTarget).parents(".apt_scrollview").scrollLeft();
                y = jQuery(event.currentTarget).parents(".apt_scrollview").scrollTop();
            }
            else
            {
                if (jQuery(event.currentTarget).find(".apt_scrollview").length > 0)
                {
                    x = jQuery(event.currentTarget).find(".apt_scrollview").scrollLeft();
                    y = jQuery(event.currentTarget).find(".apt_scrollview").scrollTop();
                }
            }

            if (event.type === "MSPointerCancel")
            {
                touch = new APT.Touch(x + event.originalEvent.pageX, y + event.originalEvent.pageY, _view);
                arrayAux[0] = touch;
            }
            else
            {
                for (var i = 0; i < event.originalEvent.targetTouches.length; i++)
                {
                    touch = new APT.Touch(x + event.originalEvent.targetTouches[i].clientX, y + event.originalEvent.targetTouches[i].clientY, _view);
                    arrayAux[i] = touch;
                }
            }

            var touches = APT.Set.NewWithObjects(arrayAux);
            var e = new APT.Event();
            e._evnt = event;
            e.setTouches(touches);
            _view.touchesCancelled_withEvent(touches, e);
            event.stopPropagation();
        });
    },

    setProperties: function() {
        $("body").append(this.jQElement());

        if (this.jQElement().attr("id") !== undefined && this.jQElement().css("left").length > 0)
        {
            var width = parseInt(window.getComputedStyle(this.jQElement()[0]).getPropertyValue("width").replace("px", ""));
            var height = parseInt(window.getComputedStyle(this.jQElement()[0]).getPropertyValue("height").replace("px", ""));
            if (APT.Global.isMemberOfClass(this, APT.ScrollView) && !navigator.msPointerEnabled)
            {
                width = width + 17;
                height = height + 17;
            }
            else
            {
                if (APT.Global.isMemberOfClass(this, APT.TextView))
                {
                    this.jQElement().css("max-height", height);
                }
            }

            var frame = APT.Rect.New(this.jQElement().css("left"), this.jQElement().css("top"), width, height);
            APT.Point.updateCSS(this.jQElement(), frame.origin);
            APT.Size.updateCSS(this.jQElement(), frame.size);

            if (APT.Global.isMemberOfClass(this, APT.Switch))
            {
                this._bounds = APT.Rect.New(0, 0, 0, 0);
            }
            else
            {
                this._bounds = this.frame();
            }
        }

        if (this.jQElement().css("background-color") !== undefined && this.jQElement().css("background-color").length > 0)
        {
            if (APT.Global.isMemberOfClass(this, APT.Button) === false)
            {
                var colAux = new APT.Color();
                colAux.updateFromCSS(this.jQElement().css("background-color"));
                this.setBackgroundColor(colAux);
            }
        }

        if (this["setFont"] instanceof Function)
        {
            if (this.jQElement().css("font").length > 0)
            {
                var fontAux = new APT.Font();
                fontAux.updateFromCSS(this.jQElement());
                (this["setFont"])(fontAux);
            }
        }

        if (this["setTextColor"] instanceof Function)
        {
            var textColorAux = new APT.Color();
            textColorAux.updateFromCSS(this.jQElement().css("color"));
            (this["setTextColor"])(textColorAux);
        }

        if (this["setTextAlignment"] instanceof Function)
        {
            if (this.jQElement().css("text-align").length > 0)
            {
                var align = APT.TextAlignment.parseCSSTextAlign(this.jQElement().css("text-align"));
                (this["setTextAlignment"])(align);
            }
        }

        if (this["setTitleColor"] instanceof Function)
        {
            var textColorAux = new APT.Color();
            textColorAux.updateFromCSS(this.jQElement().css("color"));
            (this["setTitleColor"])(textColorAux);
        }

        this.jQElement().detach();
    },

    processNode: function(markup, element) {
        var aux = element.children().clone();
        var children = element.children().detach();
        var text = null;

        if (element.text() !== undefined && element.text().trim().length > 0)
        {
            markup.text(element.text());
        }

        var i = 0;
        for (i = 0; i < children.length; i++)
        {
            if (jQuery(children[i]).text() !== undefined && jQuery(children[i]).text().trim().length > 0)
            {
                text = jQuery(children[i]).text();
                break;
            }
        }

        if (text !== undefined && markup.children()[i] !== undefined)
        {
            jQuery(markup.children()[i]).text(text);
        }

        this.setJQElement(markup);

        if (children.length > 0)
        {
            APT.Global.processElement(aux, this);
        }
    },

    setViewController: function(aViewController) {
        this._viewController = aViewController;
    },

    viewController: function() {
        return this._viewController;
    },

    indexOfView: function(aView) {
        var len = this._subviews.length;
        for (var i = 0; i < len; i++)
        {
            if (this._subviews[i] === aView)
            {
                return i;
            }
        }

        return -1;
    },

    viewWillAppear: function(animated) {
        var subviewsLength = this._subviews.length;
        var subviewsArray = this._subviews.slice(0, subviewsLength);
        for (var i = 0; i < subviewsLength; i++)
        {
            var subview = subviewsArray[i];
            subview.viewWillAppear(animated);
        }

        if (this._viewController !== null)
        {
            this._viewController.viewWillAppear(animated);
        }
    },

    viewDidAppear: function(animated) {
        var subviewsLength = this._subviews.length;
        var subviewsArray = this._subviews.slice(0, subviewsLength);
        for (var i = 0; i < subviewsLength; i++)
        {
            var subview = subviewsArray[i];
            subview.viewDidAppear(animated);
        }

        if (this._viewController !== null)
        {
            this._viewController.viewDidAppear(animated);
        }
    },

    viewWillDisappear: function(animated) {
        var subviewsLength = this._subviews.length;
        var subviewsArray = this._subviews.slice(0, subviewsLength);
        for (var i = 0; i < subviewsLength; i++)
        {
            var subview = subviewsArray[i];
            subview.viewWillDisappear(animated);
        }

        if (this._viewController !== null)
        {
            this._viewController.viewWillDisappear(animated);
        }
    },

    viewDidDisappear: function(animated) {
        var subviewsLength = this._subviews.length;
        var subviewsArray = this._subviews.slice(0, subviewsLength);
        for (var i = 0; i < subviewsLength; i++)
        {
            var subview = subviewsArray[i];
            subview.viewDidDisappear(animated);
        }

        if (this._viewController !== null)
        {
            this._viewController.viewDidDisappear(animated);
        }
    },

    removeFromSuperviewWithoutNotifying: function() {
        if (this.superview() !== null && this.superview().jQElement() !== null)
        {
            var index = this._superview.indexOfView(this);
            if (index > -1)
            {
                this._superview._subviews.splice(index, 1);
            }

            this.jQElement().detach();
            this._superview = null;
        }
    },

    HTMLCanvas: function() {
        if (this._canvas === null)
        {
            this._canvas = new APT.Canvas();
            this.setCanvas();
            this.jQElement().append(this._canvas.jQElement());
        }

        return this._canvas.jQElement()[0];
    },

    addSubviewWithoutNotifying: function(child, push) {
        this.insertSubviewAtIndexWithoutNotifying(child, this._subviews.length, push);
    },

    insertSubviewAtIndexWithoutNotifying: function(aView, anIndex, push) {
        var len = this._subviews.length;
        if (aView !== null && aView !== this && anIndex >= 0 && anIndex <= len)
        {
            if (aView.superview() !== null)
            {
                aView.removeFromSuperviewWithoutNotifying();
            }

            if (anIndex === len)
            {
                this._jqElement.append(aView.jQElement());
                this._subviews.push(aView);
            }
            else
            {
                aView.jQElement().insertBefore((this._subviews[anIndex]).jQElement());
                this._subviews.splice(anIndex, 0, aView);
            }

            aView._superview = this;
            if (push === 0)
            {
                aView.setNeedsLayout();
                aView.setNeedsDisplay();
            }
        }
    },

    addSubviewFromXibFiles: function(child) {
        this.insertSubviewAtIndexFromXibFiles(child, this._subviews.length);
    },

    insertSubviewAtIndexFromXibFiles: function(aView, anIndex) {
        this.insertSubviewAtIndexWithoutNotifying(aView, anIndex, 1);
    },

    touchesBegan_withEvent: function(touches, e) {
    },

    touchesMoved_withEvent: function(touches, e) {
    },

    touchesEnded_withEvent: function(touches, e) {
    },

    touchesCancelled_withEvent: function(touches, e) {
    },

    becomeFirstResponder: function() {
        return false;
    },

    resignFirstResponder: function() {
        return false;
    },

    isInDom: function() {
        return this.jQElement().parents("body").length === 1;
    },

    didAddSubview: function(subview) {
    },

    didMoveToSuperview: function() {
    },

    didMoveToWindow: function() {
    },

    willMoveToSuperview: function(newSuperview) {
    },

    willMoveToWindow: function(newWindow) {
    },

    willRemoveSubview: function(view) {
    },

    callSetNeedsDisplay: function(flag) {
        if (flag)
        {
            this.callDrawRect();
        }

        for (var i = 0; i < this.subviews().length; i++)
        {
            if (this.subviews()[i] !== undefined)
            {
                (this.subviews()[i]).callSetNeedsDisplay(true);
            }
        }
    },

    callDrawRect: function() {
        if (this["drawRect"] instanceof Function)
        {
            APT.Window.setCurrentGraphicsContext(this.HTMLCanvas().getContext("2d"));
            (this["drawRect"])(this.frame());
            APT.Window.popCurrentGraphicsContext();
        }
    },

});


